#include "DoraCentralDocument.h"
#include "DoraMainFrame.h"
#include "DoraCentralView.h"
#include "BioritmView.h"

DoraRuntimeClass BioritmView::classBioritmView={"BioritmView", BioritmView::CreateView, NULL};

DoraClassObject *BioritmView::CreateView(QWidget *parent)
{
  return new BioritmView(parent);
}

DoraClassObject *BioritmView::CreateDocument()
{
  return NULL;
}

void BioritmView::OnDraw(QPainter *p)
{
  p->setFont(font);
  setPixmap();
  int x1 = clientRect.x();
  int y1 = clientRect.y();
  int x2 = x1 + clientRect.width();
  int y2 = y1 + clientRect.height();
  bitBlt(this, x1, y1, &pixmap, x1, y1, x2 - x1, y2 - 1);
  m_pPushButton->show();
  m_pCheckHGroup->show();
  if(m_pDateDialog != NULL)
    m_pDateDialog->show();
}

void BioritmView::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);

  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int wf = sizeFont.width();
  int hf = fm.height() - fm.descent();
  int len = strlen(m_pPushButton->caption1.p);
  
  int x = (clientRect.width() - 9) - ((len + 2) * wf);
  int y = (clientRect.y() + (cy - (hf + 7))) - 5;
  int w = ((len + 2) * wf) + 4;
  int h = hf + 7;
  QRect buttonRect;
  buttonRect.setX(x);
  buttonRect.setY(y);
  buttonRect.setWidth(w);
  buttonRect.setHeight(h);
  m_pPushButton->setGeometry(buttonRect);
  m_pPushButton->hide();

  QRect checkGroupRect;
  checkGroupRect.setX(clientRect.x() + 4);
  checkGroupRect.setY(clientRect.y() + (cy - (hf + 7)) - 8);
  checkGroupRect.setWidth((clientRect.width() - buttonRect.width()) - 14);
  checkGroupRect.setHeight((hf + 7) + 4);
  m_pCheckHGroup->setGeometry(checkGroupRect);
  m_pCheckHGroup->hide();
  if(m_pDateDialog != NULL) {
    int wDialog = 35 * wf;
    int hDialog = (12 * hf);
    int xDialog = clientRect.x() + (int)((clientRect.width() - wDialog) / 2);
    int yDialog = clientRect.y() + (int)((clientRect.height() - hDialog) / 2);
    QRect dialogRect;
    dialogRect.setX(xDialog);
    dialogRect.setY(yDialog);
    dialogRect.setWidth(wDialog);
    dialogRect.setHeight(hDialog);
    m_pDateDialog->setGeometry(dialogRect);
    m_pDateDialog->hide();
  }
}

void BioritmView::OnCreate(DoraClassObject *pObject, DoraBaseView *pView)
{
  m_pObject = pObject;
  m_pView = pView;

  DoraCentralDocument *pDoc = GetDocument();
  DoraRuntimeClass *runtimeCheckHGroup = RUNTIME_CLASS(DoraCheckHGroup);
  DoraClassObject *objectCheckHGroup = runtimeCheckHGroup->CreateView(this);
  m_pCheckHGroup = dynamic_cast<DoraCheckHGroup *>(objectCheckHGroup);
  m_pCheckHGroup->convChar = convChar;
  m_pCheckHGroup->selectNewFont(font);
  m_pCheckHGroup->OnCreate(m_pObject, this);
  m_pCheckHGroup->setCheck(indexCicle);
  m_pCheckHGroup->hide();

  DoraRuntimeClass *runtimePushButton = RUNTIME_CLASS(DoraPushButton);
  DoraClassObject *objectPushButton = runtimePushButton->CreateView(this);
  m_pPushButton = dynamic_cast<DoraPushButton *>(objectPushButton);
  m_pPushButton->selectNewFont(font);
  m_pPushButton->convChar = convChar;
  m_pPushButton->OnCreate(m_pObject, this);
 
 if(pDoc->bDate == "") {
    m_pPushButton->ctrlID = DM_PUSHBUTTON;
    m_pPushButton->nID = ID_PUSHBUTTON_OK;
  } else {
    m_pPushButton->ctrlID = DM_PUSHBUTTON;
    m_pPushButton->nID = ID_PUSHBUTTON_EXIT;
  }
  m_pPushButton->hide();
  Str35 cap1, cap2;
  cap2 = "";cap1 = "";
  if(pDoc->bDate == "")
    cap1 = pDoc->capInfoButton1;
  else
    cap1 = pDoc->capExit;
  m_pPushButton->setCaption(cap1, cap2);
  maxDay = pDoc->maxDayMonth;
  if(pDoc->bDate == "") {
    setNewBDate();
  } else {
    calcPoint();
  }
}

void BioritmView::PostMouseMessage(int ctrl, int index)
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralView *m_pCentralView = m_pMainFrame->GetCentralView();
  if(ctrl == DM_PUSHBUTTON) {
    if(index == ID_PUSHBUTTON_EXIT)
      m_pCentralView->closeBioritm();
    if(index == ID_PUSHBUTTON_OK)
      calcPoint();
  }
  if(ctrl == DM_CHECKBUTTON) {
    indexCicle = index;
    repaint(false);
  }
}


DoraCentralDocument *BioritmView::GetDocument()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = m_pMainFrame->GetDocument();

  return pDoc;
}
	    
void BioritmView::sortPoint()
{
  int maxElement = itemPoint.maxElement();
  for(int gap = maxElement/2; gap > 0; gap /= 2) {
    for(int i = gap; i < maxElement; i++) {
      for(int j = i - gap; j >= 0; j -= gap) {
	Str35 date1, date2;
	date1 = "";date1 = itemPoint[j].mDate;
	date2 = "";date2 = itemPoint[j+gap].mDate;
	CompareDate d1(date1);
	CompareDate d2(date2);
	strPoint temp1;
	strPoint temp2;
	if(d1 <= d2)
	  break;
	temp1 = itemPoint[j];
	temp2 = itemPoint[j + gap];
	itemPoint.setElement(temp1, (j + gap) + 1);
	itemPoint.setElement(temp2, j + 1);
      }
    }
  }
}

void BioritmView::setPixmap()
{
  QPainter p;
  pixmap.resize(clientRect.width(), clientRect.height());
  pixmap.fill(QColor(255, 255, 255));
  p.begin(&pixmap);
  {
     p.setPen(QPen(QColor(0, 0, 0), 1));
     p.setBrush(QBrush(QColor(192, 192, 192)));
     p.drawRect(clientRect);

     QFontMetrics fm(font);
     QSize sizeFont = fm.size(SingleLine, "H", 1);
     int wf = sizeFont.width();
     int hf = fm.height() - fm.descent();

     int x1 = clientRect.x() + 1;
     int y1 = clientRect.y() + 1;
     int x2 = clientRect.width() - 2;
     int y2 = clientRect.height() - 2;
     p.setPen(QPen(QColor(255, 255, 255), 1));
     p.drawLine(x1, y2, x1, y1);
     p.drawLine(x1, y1, x2, y1);
     p.setPen(QPen(QColor(0, 0, 0), 1));
     p.drawLine(x1, y2, x2, y2);
     p.drawLine(x2, y2, x2, y1);

     int xl1 = x1 + 2;
     int yl1 = y1 + 2;
     int xl2 = x2 - 2;
     int yl2 = (clientRect.y() + (clientRect.height() - (hf + 7))) - 10;
     p.setPen(QPen(QColor(128, 128, 128), 1));
     p.drawLine(xl1, yl2, xl1, yl1);
     p.drawLine(xl1, yl1, xl2, yl1);
     p.setPen(QPen(QColor(255, 255, 255), 1));
     p.drawLine(xl1, yl2, xl2, yl2);
     p.drawLine(xl2, yl2, xl2, yl1);

     int xr1 = xl1 + 2;
     int yr1 = yl1 + 2;
     int xr2 = xl2 - 2;
     int yr2 = yl2 - 2;
     int wr = xr2 - xr1;
     int hr = yr2 - yr1;
     QRect bitmapRect;
     bitmapRect.setX(xr1);
     bitmapRect.setY(yr1);
     bitmapRect.setWidth(wr);
     bitmapRect.setHeight(hr);
     p.setPen(QPen(QColor(0, 0, 0), 1));
     p.setBrush(QBrush(QColor(255, 255, 255)));
     p.drawRect(bitmapRect);

     int iPoint = (int)(wr / 31);
     int xp = 20;
     for(int i = 0; i <= maxDay - 1; i++) {
       int x = xp;
       int y = (yr2 - (2 * (hf + 4))) - 2;
       int w = 4;
       int h = 4;
       QRect rect;
       rect.setX(x);
       rect.setY(y);
       rect.setWidth(w);
       rect.setHeight(h);
       p.setPen(QPen(QColor(0, 0, 0), 1));
       p.setBrush(QBrush(QColor(128, 128, 128)));
       p.drawRect(rect);
       xp += iPoint;
     }
     int xCenter = xp - iPoint;
     DoraCentralDocument *pDoc = GetDocument();
     QFont fontDay = pDoc->getDefaultFont(2);
     p.setFont(fontDay);
     QFontMetrics fmd(fontDay);
     QSize sizeFDay = fmd.size(SingleLine, "H", 1);
     int wfd = sizeFDay.width();
     xp = 16;
     int maxElement = itemPoint.maxElement();
     for(int i = 0; i <= maxElement - 1; i++) {
       Str35 cap;
       cap = "";cap = itemPoint[i].caption;
       int lenCap = strlen(cap.p);
       int xc1 = xp;
       int y = (yr2 - (hf + 4)) - 4;
       for(int j = 0; j <= lenCap - 1; j++) {
	 char *s = new char[2];
	 s[0] = '\0';
	 sprintf(s, "%c\0", cap.p[j]);
	 QString qStr;
	 qStr = "";qStr = convChar.getCharCodec(s);
	 if(itemPoint[i].isRed)
	   p.setPen(QPen(QColor(255, 0, 0), 1));
	 else
	   p.setPen(QPen(QColor(0, 0, 0), 1));
	 p.drawText(xc1, y, qStr, 1);
	 xc1 += wfd;
	 delete s;
       }
       xp += iPoint;
     }

     int xp1 = xr1 + 5;
     int yp1 = (yr1 + 2) + (hf + 6);
     int xp2 = xr2 - 5;
     int yp2 = (yr2 - (2 * (hf + 4))) - 6;
     QRect graphRect;
     graphRect.setX(xp1);
     graphRect.setY(yp1);
     graphRect.setWidth(xp2 - xp1);
     graphRect.setHeight(yp2 - yp1);
     p.setBrush(QBrush(QColor(253, 253, 253)));
     p.setPen(QPen(QColor(0, 0, 0), 1));
     p.drawRect(graphRect);

     int maxCol = (int)(wr / wf);
     p.setFont(font);
     int len = strlen(caption.p);
     int x = xr1 + ((int)((maxCol - len) / 2) * wf);
     int y = (yr1 + 1) + hf;
     for(int i = 0; i <= len - 1; i++) {
       char *s = new char[2];
       s[0] = '\0';
       sprintf(s, "%c\0", caption.p[i]);
       QString qStr;
       qStr = "";qStr = convChar.getCharCodec(s);
       p.setPen(QPen(QColor(200, 65, 117), 1));
       p.drawText(x, y, qStr, 1);
       x += wf;
       delete s;
     }
     len = strlen(strCurentMonth.p);
     x = xr1 + ((int)((maxCol - len) / 2) * wf);
     y = yr2 - 6;
     for(int i = 0; i <= len - 1; i++) {
       char *s = new char[2];
       s[0] = '\0';
       sprintf(s, "%c\0", strCurentMonth.p[i]);
       QString qStr;
       qStr = "";qStr = convChar.getCharCodec(s);
       p.setPen(QPen(QColor(200, 65, 117), 1));
       p.drawText(x, y, qStr, 1);
       x += wf;
       delete s;
     }
     xp = 16;
     int yCenter = clientRect.y() + graphRect.y() + (int)(graphRect.height() / 2);
     p.setPen(QPen(QColor(0, 0, 0), 1, DashLine));
     p.drawLine(xp, yCenter, xCenter, yCenter);
     if((indexCicle == ID_CHECKBUTTON_ALLCICLE) || (indexCicle == ID_CHECKBUTTON_PHICICLE)) {
       xp = 16;
       int yp = yCenter - (int)(itemPoint[0].yPointP);
       p.setPen(QPen(QColor(0, 0, 0), 1));
       p.moveTo(xp, yp);
       p.setBrush(QBrush(QColor(255, 255, 255)));
       p.drawEllipse(xp - 2, yp - 2, 4, 4);
       xp += iPoint;
       for(int i = 1; i <= maxElement - 1; i++) {
	 yp = yCenter - (int)(itemPoint[i].yPointP);
	 p.setPen(QPen(QColor(0, 0, 0), 1));
	 p.lineTo(xp, yp);
	 p.setPen(QPen(QColor(0, 0, 0), 1));
	 p.setBrush(QBrush(QColor(255, 255, 255)));
	 p.drawEllipse(xp - 2, yp - 2, 4, 4);
	 xp += iPoint;
       }
     }
     if((indexCicle == ID_CHECKBUTTON_ALLCICLE) || (indexCicle == ID_CHECKBUTTON_SENCICLE)) {
       xp = 16;
       int yp = yCenter - (int)(itemPoint[0].yPointS);
       p.setPen(QPen(QColor(255, 0, 0), 1));
       p.moveTo(xp, yp);
       p.setBrush(QBrush(QColor(255, 255, 255)));
       p.drawEllipse(xp - 2, yp - 2, 4, 4);
       xp += iPoint;
       for(int i = 1; i <= maxElement - 1; i++) {
	 yp = yCenter - (int)(itemPoint[i].yPointS);
	 p.setPen(QPen(QColor(255, 0, 0), 1));
	 p.lineTo(xp, yp);
	 p.setPen(QPen(QColor(255, 0, 0), 1));
	 p.setBrush(QBrush(QColor(255, 255, 255)));
	 p.drawEllipse(xp - 2, yp - 2, 4, 4);
	 xp += iPoint;
       }
     }
     if((indexCicle == ID_CHECKBUTTON_ALLCICLE) || (indexCicle == ID_CHECKBUTTON_CONCICLE)) {
       xp = 16;
       int yp = yCenter - (int)(itemPoint[0].yPointC);
       p.setPen(QPen(QColor(0, 0, 255), 1));
       p.moveTo(xp, yp);
       p.setBrush(QBrush(QColor(255, 255, 255)));
       p.drawEllipse(xp - 2, yp - 2, 4, 4);
       xp += iPoint;
       for(int i = 1; i <= maxElement - 1; i++) {
	 yp = yCenter - (int)(itemPoint[i].yPointC);
	 p.setPen(QPen(QColor(0, 0, 255), 1));
	 p.lineTo(xp, yp);
	 p.setPen(QPen(QColor(0, 0, 255), 1));
	 p.setBrush(QBrush(QColor(255, 255, 255)));
	 p.drawEllipse(xp - 2, yp - 2, 4, 4);
	 xp += iPoint;
       }
     }
  }
  p.end();
}

void BioritmView::setNewBDate()
{
  DoraCentralDocument *pDoc = GetDocument();
  DoraRuntimeClass *runtimeBox = RUNTIME_CLASS(DateDialog);
  DoraClassObject *objectBox = runtimeBox->CreateView(this);
  m_pDateDialog = dynamic_cast<DateDialog *>(objectBox);
  m_pDateDialog->selectNewFont(font);
  m_pDateDialog->convChar = convChar;
  m_pDateDialog->caption = "";m_pDateDialog->caption = pDoc->capBioDialog;
  m_pDateDialog->OnCreate(m_pObject, this);
  Str35 prefix;
  m_pDateDialog->m_pEditBox->prefix = "";
  m_pDateDialog->m_pEditBox->prefix = pDoc->itemLangName[pDoc->curentLang - 1].prefix;
  m_pDateDialog->setAttr(2);
  m_pDateDialog->hide();
}

void BioritmView::calcPoint()
{
  DoraCentralDocument *pDoc = GetDocument();
  DataRecord data;
   Str35 strDay, strMonth, strYear;
  if(m_pDateDialog != NULL) {
    m_pDateDialog->getData(&data);
    int d, m, y;
    Str35 s1, s2;
    d = atoi(data.Pole[0].p);
    if((d <= 0) || (d >= 31))
      d = pDoc->curentDay;
    m = atoi(data.Pole[1].p);
    if((m <= 0) || (m >= 12))
      m = pDoc->curentMonth;
    y = atoi(data.Pole[2].p);
    if((y <= 1900) || (y >= pDoc->curentYear))
      y = pDoc->curentYear;
    s1 = "";sprintf(s1.p, "%2d\0", d);s2 = "";s2 = spaceNull(2, s1);
    strDay   = "";strDay   = s2;
    s1 = "";sprintf(s1.p, "%2d\0", m);s2 = "";s2 = spaceNull(2, s1);
    strMonth = "";strMonth = s2;
    s1 = "";sprintf(s1.p, "%4d\0", y);s2 = "";s2 = spaceNull(4, s1);
    strYear  = "";strYear  = s2;
    bDate = "";bDate = bDate + strDay + "/" + strMonth + "/" + strYear;
    pDoc->bDate = "";pDoc->bDate = bDate;
    m_pDateDialog->hide();
    delete m_pDateDialog;
    m_pDateDialog = NULL;
    pDoc->setNewBDate();
  } else {
    bDate = "";bDate = pDoc->bDate;
  }
  int j = 0;
  for(int i = 0; i <= 34; i++) {
    if(pDoc->itemCurentMonth[i].isValid) {
      strPoint item;
      item.bDate = "";item.bDate = pDoc->bDate;
      item.mDate = "";item.mDate = pDoc->itemCurentMonth[i].dateStr;
      item.caption = "";item.caption = pDoc->itemCurentMonth[i].caption;
      item.countDay = compDate.getIntervalDate(item.bDate, item.mDate);
      item.isRed = pDoc->itemCurentMonth[i].isRed;
      double xpP = (double)(((double)(item.countDay) / 23.0) * 6.28318);
      double ypP = (double)(sin(xpP) * 57.295755);
      double xpS = (double)(((double)(item.countDay) / 28.0) * 6.28318);
      double ypS = (double)(sin(xpS) * 57.295755);
      double xpC = (double)(((double)(item.countDay) / 33.0) * 6.28318);
      double ypC = (double)(sin(xpC) * 57.295755);
      item.xPointP = xpP;
      item.yPointP = ypP;
      item.xPointS = xpS;
      item.yPointS = ypS;
      item.xPointC = xpC;
      item.yPointC = ypC;
      itemPoint.setElement(item, j + 1);
      j++;
    }
  }
  sortPoint();
  Str35 s1, s2, s3;
  int indexDay, indexMonth, curentIndex;
  s1 = "";s1 = copyStr(bDate, 1, 2);
  s2 = "";s2 = copyStr(bDate, 4, 3);
  s3 = "";s3 = copyStr(bDate, 7, 4);
  indexDay = compDate.getDayOfWeek(bDate);
  indexMonth = atoi(s2.p);
  strDay = "";strDay = pDoc->itemDayName[indexDay - 1].name;
  strMonth = "";strMonth = pDoc->itemMonthName[indexMonth - 1].name;
  caption = "";caption = caption + pDoc->capBioritm + " " + strDay + ", " + s1 + " " + strMonth + " " + s3 + " !";
  curentIndex = pDoc->curentIndex;
  s1 = "";s1 = copyStr(pDoc->itemCurentMonth[curentIndex - 1].dateStr, 4, 2);
  indexMonth = atoi(s1.p);
  strMonth = "";strMonth = pDoc->itemMonthName[indexMonth - 1].name;
  strYear = "";strYear = copyStr(pDoc->itemCurentMonth[curentIndex - 1].dateStr, 7, 4);
  strCurentMonth = "";strCurentMonth = strCurentMonth + strMonth + " " + strYear;
  repaint(false);
  m_pPushButton->ctrlID = DM_PUSHBUTTON;
  m_pPushButton->nID = ID_PUSHBUTTON_EXIT;
  Str35 cap1, cap2;
  cap2 = "";cap1 = "";cap1 = pDoc->capExit;
  m_pPushButton->setCaption(cap1, cap2);
  m_pPushButton->repaint(false);
}

void BioritmView::DestroyView()
{
  if(m_pPushButton != NULL) {
    m_pPushButton->hide();
    delete m_pPushButton;
    m_pPushButton = NULL;
  }
  if(m_pDateDialog != NULL) {
    m_pDateDialog->DestroyDialog();
    m_pDateDialog->hide();
    delete m_pDateDialog;
    m_pDateDialog = NULL;
  }
  itemPoint.removeAllElement();
  if(m_pCheckHGroup != NULL) {
    m_pCheckHGroup->DestroyView();
    m_pCheckHGroup->hide();
    delete m_pCheckHGroup;
    m_pCheckHGroup = NULL;
  }
}
