#include "DoraCentralDocument.h"
#include "DoraMainFrame.h"
#include "DateDialog.h"

DoraRuntimeClass DateDialog::classDateDialog={"DateDialog", DateDialog::CreateView, NULL};

DoraClassObject *DateDialog::CreateView(QWidget *parent)
{
  return new DateDialog(parent);
}

DoraClassObject *DateDialog::CreateDocument()
{
  return NULL;
}

void DateDialog::OnCreate(DoraClassObject *pObject, DoraBaseView *pView)
{
  m_pObject = pObject;
  m_pView   = pView;

  DoraRuntimeClass *runtimeBox = RUNTIME_CLASS(DoraEditBox);
  DoraClassObject *objectBox = runtimeBox->CreateView(this);
  m_pEditBox = dynamic_cast<DoraEditBox *>(objectBox);
  m_pEditBox->convChar = convChar;
  m_pEditBox->selectNewFont(font);
  m_pEditBox->OnCreate(m_pObject, this);
  m_pEditBox->hide();
}

void DateDialog::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);

  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int wf = sizeFont.width();
  int hf = fm.height() - fm.descent();

  int len = strlen(caption.p);
  int xTitle = clientRect.x() + 4;
  int yTitle = clientRect.y() + 4;
  int wTitle = (len * wf) + len + 11;
  int hTitle = hf + 4;
  titleRect.setX(xTitle);
  titleRect.setY(yTitle);
  titleRect.setWidth(wTitle);
  titleRect.setHeight(hTitle);

  int xBox = clientRect.x() + 4;
  int yBox = clientRect.y() + hTitle + 7;
  int wBox = (len * wf) + len + 11;
  int hBox = clientRect.height() - titleRect.height() - 10;
  boxRect.setX(xBox);
  boxRect.setY(yBox);
  boxRect.setWidth(wBox);
  boxRect.setHeight(hBox);
  m_pEditBox->setGeometry(boxRect);
  m_pEditBox->hide();

}

void DateDialog::OnDraw(QPainter *p)
{
  p->setFont(font);
  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->setBrush(QBrush(QColor(192, 192, 192)));
  p->drawRect(clientRect);

  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->setBrush(QBrush(QColor(0, 0, 255)));
  p->drawRect(titleRect);

  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int wf = sizeFont.width();
  int hf = fm.height() - fm.descent();
  if(caption != "") {
    int len = strlen(caption.p);
    int x = titleRect.x() + 3;
    int y = titleRect.y() + hf;
    for(int i = 0; i <= len - 1; i++) {
      char *s = new char[2];
      s[0] = '\0';
      sprintf(s, "%c\0", caption.p[i]);
      QString qStr;
      qStr = "";qStr = convChar.getCharCodec(s);
      p->setPen(QPen(QColor(255, 255, 255), 1));
      p->setBrush(QBrush(QColor(0, 0, 255)));
      p->drawText(x, y, qStr, 1);
      delete s;
      x += wf;
    }
  }
  m_pEditBox->show();
}

void DateDialog::setData(DataRecord data)
{
  m_pEditBox->setData(data, 1);
}

void DateDialog::getData(DataRecord *data)
{
  m_pEditBox->getData(data);
}

void DateDialog::setAttr(const int index)
{
  m_pEditBox->setDialogAttr(index);
}

DoraCentralDocument *DateDialog::GetDocument()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = m_pMainFrame->GetDocument();

  return pDoc;
}

void DateDialog::DestroyDialog()
{
  if(m_pEditBox != NULL) {
    m_pEditBox->hide();
    delete m_pEditBox;
    m_pEditBox = NULL;
  }
}
