#include "DoraCentralDocument.h"
#include "DoraMainFrame.h"
#include "DoraAbautView.h"

DoraRuntimeClass DoraAbautView::classDoraAbautView={"DoraAbautView", DoraAbautView::CreateView, NULL};

DoraClassObject *DoraAbautView::CreateView(QWidget *parent)
{
  return new DoraAbautView(parent);
}

DoraClassObject *DoraAbautView::CreateDocument()
{
  return NULL;
}

void DoraAbautView::OnDraw(QPainter *p)
{
  p->setFont(font);
  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->setBrush(QBrush(QColor(192, 192, 192)));
  p->drawRect(clientRect);

  int xc1 = clientRect.x() + 1;
  int yc1 = clientRect.y() + 1;
  int xc2 = clientRect.width() - 2;
  int yc2 = clientRect.height() - 2;

  p->setPen(QPen(QColor(255, 255, 255), 1));
  p->drawLine(xc1, yc2, xc1, yc1);
  p->drawLine(xc1, yc1, xc2, yc1);
  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->drawLine(xc1, yc2, xc2, yc2);
  p->drawLine(xc2, yc2, xc2, yc1);

  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->setBrush(QBrush(QColor(0, 0, 255)));
  p->drawRect(titleRect);

  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->setBrush(QBrush(QColor(255, 255, 255)));
  p->drawRect(textRect);

  m_pPushButton->show();

  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int wf = sizeFont.width();
  int hf = fm.height() - fm.descent();
  if(caption != "") {
    int len = strlen(caption.p);
    int x = titleRect.x() + wf;
    int y = titleRect.y() + 1 + hf;
    for(int i = 0; i <= len - 1; i++) {
      char *s = new char[2];
      s[0] = '\0';
      sprintf(s, "%c\0", caption.p[i]);
      QString qStr;
      qStr = "";qStr = convChar.getCharCodec(s);
      p->setPen(QPen(QColor(255, 255, 255)));
      p->setBrush(QBrush(QColor(0, 0, 255)));
      p->drawText(x, y, qStr);
      x += wf;
      delete s;
    }
  }

  int xt1 = titleRect.x() - 1;
  int yt1 = titleRect.y() - 1;
  int xt2 = titleRect.width() + 3;
  int yt2 = titleRect.height() + 4;

  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->drawLine(xt1, yt2, xt1, yt1);
  p->drawLine(xt1, yt1, xt2, yt1);
  p->setPen(QPen(QColor(255, 255, 255), 1));
  p->drawLine(xt1, yt2, xt2, yt2);
  p->drawLine(xt2, yt2, xt2, yt1);

  int xl1 = textRect.x() - 1;
  int yl1 = textRect.y() - 1;
  int xl2 = textRect.width() + 3;
  int yl2 = (titleRect.height() + 4) + (textRect.height() + 4);

  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->drawLine(xl1, yl2, xl1, yl1);
  p->drawLine(xl1, yl1, xl2, yl1);
  p->setPen(QPen(QColor(255, 255, 255), 1));
  p->drawLine(xl1, yl2, xl2, yl2);
  p->drawLine(xl2, yl2, xl2, yl1);

  DoraCentralDocument *pDoc = GetDocument();
  int maxElement = pDoc->itemStrAbaut.maxElement();
  int y = textRect.y() + hf + 20;
  for(int i = 0; i <= maxElement - 1; i++) {
    int x = textRect.x() + wf;
    CString strItem;
    strItem = "";
    strItem = pDoc->itemStrAbaut[i].strItem.p;
    int len = strlen(strItem.p);
    for(int j = 0; j <= len - 1; j++) {
      char *s = new char[2];
      s[0] = '\0';
      sprintf(s, "%c\0", strItem.p[j]);
      QString qStr;
      qStr = "";qStr = convChar.getCharCodec(s);
      p->setPen(QPen(QColor(0, 0, 0), 1));
      p->setBrush(QBrush(QColor(255, 255, 255)));
      p->drawText(x, y, qStr);
      x += wf;
      delete s;
    }
    y += (hf + 2);
  }
}

void DoraAbautView::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);

  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int wf = sizeFont.width();
  int hf = fm.height() - fm.descent();

  titleRect.setX(clientRect.x() + 4);
  titleRect.setY(clientRect.y() + 4);
  titleRect.setWidth(clientRect.width() - 9);
  titleRect.setHeight(hf + 6);

  textRect.setX(clientRect.x() + 4);
  textRect.setY(clientRect.y() + titleRect.height() + 7);
  textRect.setWidth(clientRect.width() - 9);
  textRect.setHeight(clientRect.height() - 50);

  int len = strlen(m_pPushButton->caption1.p);
  buttonRect.setX((clientRect.width() - 9) - ((len + 2) * wf));
  buttonRect.setY(clientRect.y() + titleRect.height() + 5 + textRect.height() + 4);
  buttonRect.setWidth(((len + 2) * wf) + 4);
  buttonRect.setHeight(hf + 7);
  m_pPushButton->setGeometry(buttonRect);
  m_pPushButton->hide();
}

void DoraAbautView::OnCreate(DoraClassObject *pObject, DoraBaseView *pView)
{
  m_pObject = pObject;
  m_pView   = pView;

  DoraCentralDocument *pDoc = GetDocument();

  DoraRuntimeClass *runtimePushButton = RUNTIME_CLASS(DoraPushButton);
  DoraClassObject *objectPushButton = runtimePushButton->CreateView(this);
  m_pPushButton = dynamic_cast<DoraPushButton *>(objectPushButton);
  m_pPushButton->selectNewFont(font);
  m_pPushButton->convChar = convChar;
  m_pPushButton->OnCreate(m_pObject, this);
  m_pPushButton->hide();
  m_pPushButton->ctrlID = DM_PUSHBUTTON;
  m_pPushButton->nID = ID_PUSHBUTTON_EXIT;
  Str35 cap1, cap2;
  cap1 = "";cap1 = pDoc->capExit;
  cap2 = "";
  m_pPushButton->setCaption(cap1, cap2);
  caption = "";caption = pDoc->capAbautView;
  Str35 prefix;
  prefix = "";prefix = pDoc->itemLangName[pDoc->curentLang - 1].prefix;
  pDoc->setAbautStr(prefix);
}

void DoraAbautView::PostMouseMessage(int ctrl, int index)
{
  if(m_pPushButton != NULL) {
    m_pPushButton->hide();
    delete m_pPushButton;
    m_pPushButton = NULL;
  }
  int ctrlID = ID_SIGNAL_CLOSE_ABAUTVIEW;
  if(m_pView != NULL)
    m_pView->PostMouseMessage(ctrlID, index);
}

DoraCentralDocument *DoraAbautView::GetDocument()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = m_pMainFrame->GetDocument();

  return pDoc;
}
