#if !defined(_DORABASEVIEW_H)
#define _DORABASEVIEW_H

#include <qapplication.h>
#include <qwidget.h>
#include <qevent.h>
#include <qpainter.h>
#include <qrect.h>
#include <qcolor.h>
#include <qfont.h>
#include <qfontmetrics.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <typeinfo>

#include "Typedef.h"
#include "Dynacolect.h"
#include "DoraClassObject.h"
#include "DoraConvChar.h"

class DoraBaseView : public QWidget, public DoraClassObject
{
public:
  DoraRuntimeClass *GetRuntimeClass() const {
    return &classDoraBaseView;
  }
  static DoraRuntimeClass classDoraBaseView;
  static DoraClassObject *CreateView(QWidget *parent);
  static DoraClassObject *CreateDocument();

public:
  DoraBaseView(QWidget *parent, const char *name = 0) : QWidget(parent, name), DoraClassObject() {
    printf("   DoraBaseView\n");
    LeftButtonPush = RightButtonPush = MidthButtonPush = FALSE;
  }
  ~DoraBaseView() {
    printf("   DoraBaseView\n");
  }

private:
  void paintEvent(QPaintEvent *e);
  void resizeEvent(QResizeEvent *e);
  void mousePressEvent(QMouseEvent *e);
  void mouseMoveEvent(QMouseEvent *e);
  void mouseReleaseEvent(QMouseEvent *e);
  void showEvent(QShowEvent *e);
  void hideEvent(QHideEvent *e);
  void focusInEvent(QFocusEvent *e);
  void focusOutEvent(QFocusEvent *e);
  void closeEvent(QCloseEvent *e);

public:
  virtual void OnDraw(QPainter *p);
  virtual void OnSize(int cx, int cy);
  virtual void OnInitialUpdate();
  virtual void OnUpdateView();
  virtual void OnShowView();
  virtual void OnHideView();
  virtual void OnFocus();
  virtual void OffFocus();
  virtual void DestroyView();
  virtual void OnCloseView();

public:
  virtual void PostMouseMove(int index);
  virtual void PostMouseClick(int index);
  virtual void PostMouseRelease(int index);
  virtual void PostButtonMessage(int indexButton);
  virtual void PostPushButtonMessage(int buttonID);
  virtual void PostEditMessage(const int indexEdit);
  virtual void PostKeyPressMessage(int index, int keyCode);
  virtual void PostKeyStatMessage(int stat, int keyCode);
  virtual void PostToolBarButtonMessage(const int ViewMessage, const int idToolBarButton);
  virtual void PostMouseMessage();
  virtual void PostMouseMessage(int ctrl, int index);
  virtual void MouseMove();  
  virtual void MouseRelease();
  virtual void UpdateTimeView();
  virtual void UpdateConvChar();


public:
  int   xMovePos;
  int   yMovePos;
  int   xPosView;
  int   yPosView;
  bool  LeftButtonPush;
  bool  RightButtonPush;
  bool  MidthButtonPush;
  QFont font;
  DoraConvChar convChar;

public:
  void selectNewFont(QFont fn);

};

#endif





