#include "DoraCentralDocument.h"
#include "DoraMainFrame.h"
#include "DoraCentralView.h"

DoraRuntimeClass DoraCentralView::classDoraCentralView = {"DoraCentralView", DoraCentralView::CreateView, NULL};

DoraClassObject *DoraCentralView::CreateView(QWidget *parent)
{
  return new DoraCentralView(parent);
}

DoraClassObject *DoraCentralView::CreateDocument()
{
  return NULL;
}

void DoraCentralView::OnDraw(QPainter *p)
{
  p->setFont(font);
  p->setBrush(QBrush(QColor(192, 192, 192)));
  p->setPen(QPen(QColor(192, 192, 192), 1));
  p->drawRect(clientRect);

  m_pCalendarView->show();
  m_pLeftView->show();

}

void DoraCentralView::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);

  int wView = (int)(cx / 2) - 2;
  int hView = cy - 5;

  QRect calendarRect;
  calendarRect.setX(2);
  calendarRect.setY(3);
  calendarRect.setWidth(wView - 2);
  calendarRect.setHeight(hView);
  m_pCalendarView->setGeometry(calendarRect);
  m_pCalendarView->hide();

  QRect leftRect;
  leftRect.setX(5 + (wView - 2));
  leftRect.setY(3);
  leftRect.setWidth(wView - 2);
  leftRect.setHeight(hView);
  m_pLeftView->setGeometry(leftRect);
  m_pLeftView->hide();
}

void DoraCentralView::OnCreate(DoraClassObject *pObject)
{
  setFont(font);
  m_pObject = pObject;
  DoraCentralDocument *pDoc = GetDocument();

  DoraRuntimeClass *runtimeCalendar = RUNTIME_CLASS(DoraCalendarView);
  DoraClassObject *objectCalendar = runtimeCalendar->CreateView(this);
  m_pCalendarView = dynamic_cast<DoraCalendarView *>(objectCalendar);
  m_pCalendarView->selectNewFont(pDoc->getDefaultFont(5));
  m_pCalendarView->convChar = pDoc->convChar;
  m_pCalendarView->OnCreate(m_pObject, this);
  m_pCalendarView->hide();

  DoraRuntimeClass *runtimeLeft = RUNTIME_CLASS(DoraLeftView);
  DoraClassObject *objectLeft = runtimeLeft->CreateView(this);
  m_pLeftView = dynamic_cast<DoraLeftView *>(objectLeft);
  m_pLeftView->convChar = pDoc->convChar;
  m_pLeftView->selectNewFont(pDoc->getDefaultFont(5));
  m_pLeftView->OnCreate(m_pObject, this);
  m_pLeftView->hide();
  
  DoraRuntimeClass *runtimeDialogDoc = RUNTIME_CLASS(DoraDialogDoc);
  DoraClassObject *objectDialogDoc = runtimeDialogDoc->CreateDocument();
  m_pDialogDoc = dynamic_cast<DoraDialogDoc *>(objectDialogDoc);
  m_pDialogDoc->initDocument();

  /*
  DoraRuntimeClass *runtimeBioritmView = RUNTIME_CLASS(BioritmView);
  DoraClassObject  *objectBioritmView = runtimeBioritmView->CreateView(this);
  m_pBioritmView = dynamic_cast<BioritmView *>(objectBioritmView);
  m_pBioritmView->convChar = convChar;
  m_pBioritmView->selectNewFont(font);
  m_pBioritmView->OnCreate(m_pObject, this);
  m_pBioritmView->hide();
  */
}

void DoraCentralView::DestroyView()
{
}

DoraCentralDocument *DoraCentralView::GetDocument()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = m_pMainFrame->GetDocument();  
  return pDoc;
}

void DoraCentralView::stopClock()
{
  m_pLeftView->stopClock();
}

void DoraCentralView::startClock()
{
  m_pLeftView->startClock();
}

void DoraCentralView::setCurentDate()
{
  m_pCalendarView->updateView();
}

DoraDialogDoc *DoraCentralView::GetDialogDoc()
{
  return m_pDialogDoc;
}

void DoraCentralView::UpdateConvChar()
{
  m_pCalendarView->convChar = convChar;
  m_pCalendarView->UpdateConvChar();
  m_pLeftView->convChar = convChar;
  m_pLeftView->UpdateConvChar();
}

void DoraCentralView::setBioritms()
{
  if(m_pBioritmView != NULL) {
    m_pBioritmView->hide();
    delete m_pBioritmView;
    m_pBioritmView = NULL;
  }

  DoraRuntimeClass *runtimeBioritmView = RUNTIME_CLASS(BioritmView);
  DoraClassObject  *objectBioritmView = runtimeBioritmView->CreateView(this);
  m_pBioritmView = dynamic_cast<BioritmView *>(objectBioritmView);
  m_pBioritmView->convChar = convChar;
  m_pBioritmView->selectNewFont(font);
  m_pBioritmView->OnCreate(m_pObject, this);
  m_pBioritmView->hide();
  
  int w = (int)(3 * (clientRect.width() / 3)) - 10;
  int h = (int)(3 * (clientRect.height() / 3)) - 5;

  int x = (int)((clientRect.x() + (clientRect.width() - w)) / 2);
  int y = (int)((clientRect.y() + (clientRect.height() - h)) / 2);

  QRect rect;
  rect.setX(x);
  rect.setY(y);
  rect.setWidth(w);
  rect.setHeight(h);

  m_pBioritmView->setGeometry(rect);
  m_pBioritmView->show();
}

void DoraCentralView::closeBioritm()
{
  if(m_pBioritmView != NULL) {
    m_pBioritmView->hide();
    delete m_pBioritmView;
    m_pBioritmView = NULL;
  }
}



