#include "DoraCentralDocument.h"
#include "DoraCentralView.h"
#include "DoraMainFrame.h"
#include "DoraCheckGroup.h"

DoraRuntimeClass DoraCheckGroup::classDoraCheckGroup={"DoraCheckGroup", DoraCheckGroup::CreateView, NULL};

DoraClassObject *DoraCheckGroup::CreateView(QWidget *parent)
{
  return new DoraCheckGroup(parent);
}

DoraClassObject *DoraCheckGroup::CreateDocument()
{
  return NULL;
}

void DoraCheckGroup::OnDraw(QPainter *p)
{
  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->setBrush(QBrush(QColor(192, 192, 192)));
  p->drawRect(clientRect);

  int x1 = clientRect.x() + 1;
  int y1 = clientRect.y() + 1;
  int x2 = clientRect.width() - 2;
  int y2 = clientRect.height() - 2;

  p->setFont(font);
  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->drawLine(x1, y1, x2, y1);
  p->drawLine(x1, y2, x1, y1);
  p->setPen(QPen(QColor(255, 255, 255), 1));
  p->drawLine(x1, y2, x2, y2);
  p->drawLine(x2, y2, x2, y1);

  int maxElement = itemCheckGroup.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    itemCheckGroup[i].checkButton->show();
    itemCheckGroup[i].staticCtrl->show();
  }
  
  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int w = sizeFont.width();
  int h = fm.height() - fm.descent() + 4;
  p->setFont(font);

  if(capGroup != "") {
    p->setPen(QPen(QColor(0, 0, 0), 1));
    p->setBrush(QBrush(QColor(192, 192, 192)));
    int xGroup = clientRect.x() + 3;
    int len = strlen(capGroup.p);
    int x = xGroup + w;
    int y = clientRect.y() + h;
    for(int i = 0; i <= len - 1; i++) {
      char *s = new char[2];
      s[0] = '\0';
      sprintf(s, "%c\0", capGroup.p[i]);
      QString qStr;
      qStr = "";qStr = convChar.getCharCodec(s);
      p->drawText(x, y, s);
      x += w;
      delete s;
    }
  }
}

void DoraCheckGroup::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);
  
  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int w = (int)((cx - 4) / 2);
  int h = fm.height() - fm.descent() + 4;
  int maxElement = itemCheckGroup.maxElement();
  int y = 4;
  if(capGroup != "")
    y = clientRect.y() + h + 4;
  for(int i = 0; i <= maxElement - 1; i++) {
    int m = (i + 1) % 2;
    int x = 4;
    if(m == 0)
      x += (w + 1);;
    if(((i + 1) > 2) && (m != 0))
      y += h;
    setSizeItems(i + 1, x, y, w, h);
  }
}

void DoraCheckGroup::OnCreate(DoraClassObject *pObject, DoraBaseView *pView)
{
  m_pObject = pObject;
  m_pView   = pView;
  setFont(font);
}

void DoraCheckGroup::setGroupItems(ItemCheckGroup x, const int index)
{
  ItemCheckGroup item;
  item = NULL;
  item = x;
  itemCheckGroup.setElement(item, index);
}

void DoraCheckGroup::createGroup(const int index, const Str35 caption)
{
  QColor textColor(0, 0, 0);
  QColor backColor(192, 192, 192);
  QColor bordColor(192, 192, 192);
  ItemCheckGroup item;
  item = NULL;

  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralView *m_pCentralView = m_pMainFrame->GetCentralView();
  DoraCentralDocument *pCentralDoc = m_pCentralView->GetDocument();
  
  DoraRuntimeClass *runtimeCheckButton = RUNTIME_CLASS(DoraCheckButton);
  DoraClassObject  *objectCheckButton  = runtimeCheckButton->CreateView(this);
  item.checkButton = dynamic_cast<DoraCheckButton *>(objectCheckButton);
  item.checkButton->selectNewFont(font);
  item.checkButton->convChar = pCentralDoc->convChar;
  item.checkButton->OnCreate(m_pObject, this, index, ID_CHECK_BUTTON_CHECKER);
  item.checkButton->hide();

  DoraRuntimeClass *runtimeStaticCtrl = RUNTIME_CLASS(DoraStaticCtrl);
  DoraClassObject  *objectStaticCtrl  = runtimeStaticCtrl->CreateView(this);
  item.staticCtrl = dynamic_cast<DoraStaticCtrl *>(objectStaticCtrl);
  item.staticCtrl->selectNewFont(font);
  item.staticCtrl->convChar = pCentralDoc->convChar;
  item.staticCtrl->OnCreate(m_pObject, this);
  item.staticCtrl->setColor(textColor, bordColor, backColor);
  item.staticCtrl->setCaption(caption);
  item.staticCtrl->nID = index;
  item.staticCtrl->hide();

  setGroupItems(item, index);
}

void DoraCheckGroup::setSizeItems(const int index, const int x1, const int y1, const int w, const int h)
{
  ItemCheckGroup item;
  item = NULL;
  item = itemCheckGroup[index - 1];
  
  QFontMetrics fm(font);
  QSize sFont = fm.size(SingleLine, "H", 1);

  int xb1 = x1;
  int yb1 = y1;
  int xb2 = x1 + h;
  int yb2 = y1 + h;

  QRect rectB;
  rectB.setX(xb1);
  rectB.setY(yb1);
  rectB.setWidth(xb2 - xb1);
  rectB.setHeight(yb2 - yb1);
  item.checkButton->setGeometry(rectB);
  item.checkButton->hide();

  int xs1 = x1 + h + 2;
  int ys1 = y1;
  int xs2 = xs1 + ((int)(10 * sFont.width()) + 11);
  int ys2 = y1 + h;

  QRect rectS;
  rectS.setX(xs1);
  rectS.setY(ys1);
  rectS.setWidth(xs2 - xs1);
  rectS.setHeight(ys2 - ys1);
  item.staticCtrl->setGeometry(rectS);
  item.staticCtrl->hide();

  setGroupItems(item, index);
}

void DoraCheckGroup::PostMouseMessage(int ctrl, int index)
{
  int maxElement = itemCheckGroup.maxElement();
  if((index >= 1) && (index <= maxElement)) {
    setCheckButton(index);
    m_pView->PostMouseMessage(POST_CHECK_GROUP, index);
  }
}

void DoraCheckGroup::setCheckButton(const int index)
{
  int maxElement = itemCheckGroup.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    ItemCheckGroup item;
    item = itemCheckGroup[i];
    if(index == i + 1)
      item.checkButton->setCheck(true);
    else
      item.checkButton->setCheck(false);
    itemCheckGroup.setElement(item, i + 1);
  }

}

void DoraCheckGroup::UpdateConvChar()
{
  int maxElement = itemCheckGroup.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    ItemCheckGroup item;
    item = itemCheckGroup[i];
    item.checkButton->convChar = convChar;
    item.checkButton->repaint(false);
    item.staticCtrl->convChar = convChar;
    item.staticCtrl->repaint(false);
    itemCheckGroup.setElement(item, i + 1);
  }

  repaint(false);
}

void DoraCheckGroup::setCaptions(const int index, const Str35 caption)
{
  int maxElement = itemCheckGroup.maxElement();
  if((index >= 1) && (index <= maxElement)) {
    ItemCheckGroup item;
    item = itemCheckGroup[index - 1];
    item.staticCtrl->setCaption(caption);
    itemCheckGroup.setElement(item, index);
  }
}
