#include "DoraCentralDocument.h"
#include "DoraCentralView.h"
#include "DoraMainFrame.h"
#include "DoraCheckHGroup.h"

DoraRuntimeClass DoraCheckHGroup::classDoraCheckHGroup={"DoraCheckHGroup", DoraCheckHGroup::CreateView, NULL};

DoraClassObject *DoraCheckHGroup::CreateView(QWidget *parent)
{
  return new DoraCheckHGroup(parent);
}

DoraClassObject *DoraCheckHGroup::CreateDocument()
{
  return NULL;
}

void DoraCheckHGroup::OnCreate(DoraClassObject *pObject, DoraBaseView *pView)
{
  m_pObject = pObject;
  m_pDialog = NULL;
  m_pView   = pView;
  setCheckButton();
}

void DoraCheckHGroup::OnCreate(DoraClassObject *pObject, DoraBaseDialog *pDialog)
{
  m_pObject = pObject;
  m_pDialog = pDialog;
  m_pView   = NULL;
  setCheckButton();
}

void DoraCheckHGroup::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);
  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int wf = sizeFont.width();
  int hf = fm.height() - fm.descent();
  int x = clientRect.x() + 2;
  int y = clientRect.y() + 1;
  int w = 21;
  int h = 21;
  int maxElement = itemButton.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    int len = strlen(itemButton[i].cap1.p);
    QRect rect;
    rect.setX(x);
    rect.setY(y);
    rect.setWidth(w);
    rect.setHeight(h);
    strButton item;
    item = itemButton[i];
    item.button->setGeometry(rect);
    item.button->hide();
    item.xb = x;
    item.yb = y;
    item.wb = w;
    item.hb = h;
    itemButton.setElement(item, i + 1);
    x += (w + ((len + 1) * wf) + 4);
  }
}

void DoraCheckHGroup::OnDraw(QPainter *p)
{
  p->setFont(font);
  p->setPen(QPen(QColor(192, 192, 192), 1));
  p->setBrush(QBrush(QColor(192, 192, 192)));
  p->drawRect(clientRect);
  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int wf = sizeFont.width();
  int hf = fm.height() - fm.descent();
  int maxElement = itemButton.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    itemButton[i].button->show();
    int x = itemButton[i].xb + itemButton[i].wb + 4;
    int y = (itemButton[i].yb + hf) - 2;
    int lenCap1 = strlen(itemButton[i].cap1.p);
    int lenCap2 = strlen(itemButton[i].cap2.p);
    for(int j = 0; j <= lenCap1 - 1; j++) {
      char *s = new char[2];
      s[0] = '\0';
      sprintf(s, "%c\0", itemButton[i].cap1.p[j]);
      QString qStr;
      qStr = "";qStr = convChar.getCharCodec(s);
      p->setPen(QPen(QColor(0, 0, 0), 1));
      p->setBrush(QBrush(QColor(192, 192, 192)));
      p->drawText(x, y, qStr, 1);
      x += wf;
      delete s;
    }
    x = itemButton[i].xb + itemButton[i].wb + 4;
    y += (hf - 4);
    for(int j = 0; j <= lenCap2 - 1; j++) {
      char *s = new char[2];
      s[0] = '\0';
      sprintf(s, "%c\0", itemButton[i].cap2.p[j]);
      QString qStr;
      qStr = "";qStr = convChar.getCharCodec(s);
      p->setPen(QPen(QColor(0, 0, 0), 1));
      p->setBrush(QBrush(QColor(192, 192, 192)));
      p->drawText(x, y, qStr, 1);
      x += wf;
      delete s;
    }
  }
}

void DoraCheckHGroup::DestroyView()
{
  int maxElement = itemButton.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    DoraCheckBmpButton *button;
    button = itemButton[i].button;
    delete button;
    button = NULL;
  }

}

void DoraCheckHGroup::PostMouseMessage(int ctrl, int index)
{
  int maxElement = itemButton.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    strButton item;
    item = itemButton[i];
    item.button->setCheck(false);
    if(item.button->nID == index)
      item.button->setCheck(true);
    itemButton.setElement(item, i + 1);
  }
  if(m_pView != NULL)
    m_pView->PostMouseMessage(ctrl, index);
  if(m_pDialog != NULL)
    m_pDialog->PostMouseMessage(ctrl, index);
}

void DoraCheckHGroup::setCheck(const int nID)
{
  int maxElement = itemButton.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    strButton item;
    item = itemButton[i];
    item.button->setCheck(false);
    if(item.button->nID == nID)
      item.button->setCheck(true);
    itemButton.setElement(item, i + 1);
  }
}

DoraCentralDocument *DoraCheckHGroup::GetDocument()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = m_pMainFrame->GetDocument();
  return pDoc;
}

void DoraCheckHGroup::setCheckButton()
{
  DoraCentralDocument *pDoc = GetDocument();
  Str35 prefix;
  prefix = "";prefix = pDoc->itemLangName[pDoc->curentLang-1].prefix;
  attr.setToolCheck(prefix);
  int maxElement = attr.itemAttr.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    strButton item;
    DoraRuntimeClass *runtimeButton = RUNTIME_CLASS(DoraCheckBmpButton);
    DoraClassObject *objectButton = runtimeButton->CreateView(this);
    item.button = dynamic_cast<DoraCheckBmpButton *>(objectButton);
    item.button->convChar = convChar;
    item.button->selectNewFont(font);
    item.cap1 = "";item.cap1 = attr.itemAttr[i].cap1;
    item.cap2 = "";item.cap2 = attr.itemAttr[i].cap2;
    item.button->ctrlID = attr.itemAttr[i].ctrlID;
    item.button->nID = attr.itemAttr[i].nID;
    item.button->OnCreate(m_pObject, this);
    item.button->hide();
    itemButton.setElement(item, i + 1);
  }
}
