#include "DoraCentralDocument.h"
#include "DoraCentralView.h"
#include "DoraMainFrame.h"

DoraRuntimeClass DoraClockView::classDoraClockView={"DoraClockView", DoraClockView::CreateView, NULL};

DoraClassObject *DoraClockView::CreateView(QWidget *parent)
{
  return new DoraClockView(parent);
}

DoraClassObject *DoraClockView::CreateDocument()
{
  return NULL;
}

void DoraClockView::OnDraw(QPainter *p)
{
  p->setFont(font);
  setPixmap();
  setCPixmap();
  setSPixmap();
  int x1 = clientRect.x();
  int y1 = clientRect.y();
  int x2 = x1 + clientRect.width();
  int y2 = y1 + clientRect.height();
  bitBlt(this, x1, y1, &pixmap, x1, y1, x2 - x1, y2 - 1);
}

void DoraClockView::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);

  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int w = sizeFont.width();
  int h = fm.height() - fm.descent();
  int xCenter = (int)(clientRect.width() / 2);
  int yCenter = (int)(clientRect.height() / 2);
  int radius = ((int)(clientRect.height() / 2)) - 15;
  int tRadius = ((int)(clientRect.height() / 2)) - 25;
  int sRadius = ((int)(clientRect.height() / 2)) - 85;
  int hRadius = ((int)(clientRect.height() / 2)) - 33;
  int k = 0;
  for(int min = 1; min <= 60; min++) {
     pointClock item;
     double nRadians = (double)(min * 6) * 0.017453292;
     int xc = xCenter + (int)(radius * sin(nRadians));
     int yc = yCenter - (int)(radius * cos(nRadians));
     int xs = xCenter + (int)(sRadius * sin(nRadians));
     int ys = yCenter - (int)(sRadius * cos(nRadians));
     int xh = xCenter + (int)(hRadius * sin(nRadians));
     int yh = yCenter - (int)(hRadius * cos(nRadians));
     item.xPos = xc;
     item.yPos = yc;
     itemPointClock.setElement(item, min);
     item.xPos = xs;
     item.yPos = ys;
     itemSmallClock.setElement(item, min);
     item.xPos = xh;
     item.yPos = yh;
     itemHourClock.setElement(item, min);
     textRect itemRect;
     int xt = xCenter + (int)(tRadius * sin(nRadians));
     int yt = yCenter - (int)(tRadius * cos(nRadians));
     int wt = w;
     int ht = h;
     QRect rect;
     rect.setX(xt - 5);
     rect.setY(yt - 4);
     rect.setWidth(2 * wt);
     rect.setHeight(h);
     itemRect.rect.setX(rect.x());
     itemRect.rect.setY(rect.y());
     itemRect.rect.setWidth(rect.width());
     itemRect.rect.setHeight(rect.height());
     itemTextRect.setElement(itemRect, min);
     k++;
  }
}

void DoraClockView::OnCreate(DoraClassObject *pObject, DoraBaseView *pView)
{
  m_pObject = pObject;
  m_pView   = pView;
  setFont(font);
}

DoraCentralDocument *DoraClockView::GetDocument()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralView *m_pCentralView = m_pMainFrame->GetCentralView();
  DoraCentralDocument *pDoc = m_pCentralView->GetDocument();
  return pDoc;
}

void DoraClockView::setPixmap()
{
  QPainter p;
  pixmap.resize(clientRect.width(), clientRect.height());
  pixmap.fill(QColor(255, 255, 255));
  p.begin(&pixmap);
  {
    p.setFont(font);
    p.setPen(QPen(QColor(0, 0, 0), 1));
    p.setBrush(QBrush(QColor(192, 192, 192)));
    p.drawRect(clientRect);
    int x = clientRect.x() + 2;
    int y = clientRect.y() + 2;
    int w = clientRect.width() - 4;
    int h = clientRect.height() - 4;
   
    int x1 = clientRect.x() + 1;
    int y1 = clientRect.y() + 1;
    int x2 = clientRect.width() - 2;
    int y2 = clientRect.height() - 2;
    p.setPen(QPen(QColor(0, 0, 0), 2));
    p.drawLine(x1, y2, x1, y1);
    p.drawLine(x1, y1, x2, y1);
    p.setPen(QPen(QColor(255, 255, 255), 2));
    p.drawLine(x1, y2, x2, y2);
    p.drawLine(x2, y2, x2, y1);

    p.setPen(QPen(QColor(0, 0, 0), 1));
    p.setBrush(QBrush(QColor(255, 255, 255)));
    QRect rect;
    rect.setX(x);
    rect.setY(y);
    rect.setWidth(w);
    rect.setHeight(h);
    p.drawRect(rect);
  }
  p.end();
}

void DoraClockView::setCPixmap()
{
  QPainter p;
  p.begin(&pixmap);
  {
    int xCenter = (int)(clientRect.width() / 2);
    int yCenter = (int)(clientRect.height() / 2);
    int step = 1;
    for(int min = 1; min <= 60; min++) {
      p.setPen(QPen(QColor(0, 0, 0), 2));
      int xc = itemPointClock[min - 1].xPos;
      int yc = itemPointClock[min - 1].yPos;
      int xs = itemSmallClock[min - 1].xPos;
      int ys = itemSmallClock[min - 1].yPos;
      int xh = itemHourClock[min - 1].xPos;
      int yh = itemHourClock[min - 1].yPos;
      p.drawPoint(xc, yc);
      if(step == 5) {
	QRect rect;
	rect.setX(xc - 2);
	rect.setY(yc - 2);
	rect.setWidth(5);
	rect.setHeight(5);
	p.setPen(QPen(QColor(0, 0, 0), 1));
	p.setBrush(QBrush(QColor(192, 192, 192)));
	p.drawRect(rect);
	step = 0;
      }
      step++;
    }
    step = 1;
    int hour = 1;
    p.setFont(font);
    for(int min = 1; min <= 60; min++) {
      if(step == 5) {
	p.setPen(QPen(QColor(0, 0, 0), 1));
	Str35 str1, str2;
	str1 = "";
	sprintf(str1.p, "%2d\0", hour);
	str2 = "";
	str2 = spaceNull(2, str1);
	QString qStr;
	qStr = "";qStr = convChar.getCharCodec(str2.p);
	p.setFont(font);
	p.drawText(itemTextRect[min-1].rect, Qt::AlignCenter, qStr);
	step = 0;hour++;
      }
      step++;
    }
  }
  p.end();
}

void DoraClockView::setSPixmap()
{
  DoraCentralDocument *pDoc = GetDocument();
  if(!killedTimer)
    pDoc->getCurentTime();
  int xCenter = (int)(clientRect.width() / 2);
  int yCenter = (int)(clientRect.height() / 2);
  int h = pDoc->curentHour;
  int m = pDoc->curentMin;
  int s = pDoc->curentSec;
  Str35 strH1, strH2, strM1, strM2, strS1, strS2, strTime;
  strH1 = "";sprintf(strH1.p, "%2d\0", h);strH2 = "";strH2 = spaceNull(2, strH1);
  strM1 = "";sprintf(strM1.p, "%2d\0", m);strM2 = "";strM2 = spaceNull(2, strM1);
  strS1 = "";sprintf(strS1.p, "%2d\0", s);strS2 = "";strS2 = spaceNull(2, strS1);
  strTime = "";strTime = strTime + strH2+":"+strM2+":"+strS2;
  QPainter p;
  p.begin(&pixmap);
  {
    p.setFont(font);
    p.setPen(QPen(QColor(0, 0, 0), 1));
    QFontMetrics fmDigital(pDoc->getDefaultFont(3));
    QSize sizeDig = fmDigital.size(SingleLine, "H", 1);
    int wd = sizeDig.width();
    int hd = fmDigital.height() - fmDigital.descent();
    int wDigital = (10 * wd) + 5;
    int hDigital = hd + 4;
    int xDigital = xCenter - (int)(wDigital / 2);
    int yDigital = yCenter + 43;
    digRect.setX(xDigital);
    digRect.setY(yDigital);
    digRect.setWidth(wDigital);
    digRect.setHeight(hDigital);
    p.setPen(QPen(QColor(0, 0, 0), 1));
    p.setBrush(QBrush(QColor(255, 255, 255)));
    p.drawRect(digRect);
    p.setFont(pDoc->getDefaultFont(3));
    p.drawText(digRect, Qt::AlignCenter, strTime.p);

    int hIndex, mIndex, sIndex;
    int smallhIndex, smallmIndex, smallsIndex;; 

    if(h > 12)
      h -= 12;
    if(!killedTimer)
      hIndex = (h * 5) + (int)(m / 10);
    else
      hIndex = h * 5;
    if(hIndex == 0)
      hIndex = 60;
    if(hIndex > 60)
      hIndex -= 60;
    mIndex = m;
    if(m == 0)
      mIndex = 60;
    sIndex = s;
    if(s == 0)
      sIndex = 60;
    p.setPen(QPen(QColor(0, 0, 0), 3));
    p.drawLine(xCenter, yCenter, itemHourClock[hIndex-1].xPos, itemHourClock[hIndex-1].yPos);
    p.drawLine(xCenter, yCenter, itemPointClock[mIndex-1].xPos, itemPointClock[mIndex-1].yPos);
    p.setPen(QPen(QColor(255, 0, 0), 2));
    p.drawLine(xCenter, yCenter, itemPointClock[sIndex-1].xPos, itemPointClock[sIndex-1].yPos);

    if(hIndex <= 30)
      smallhIndex = hIndex + 30;
    else
      smallhIndex = hIndex - 30;
    p.setPen(QPen(QColor(0, 0, 0), 3));
    p.drawLine(xCenter, yCenter, itemSmallClock[smallhIndex-1].xPos, itemSmallClock[smallhIndex-1].yPos);

    if(mIndex <= 30)
      smallmIndex = mIndex + 30;
    else
      smallmIndex = mIndex - 30;
    p.setPen(QPen(QColor(0, 0, 0), 3));
    p.drawLine(xCenter, yCenter, itemSmallClock[smallmIndex-1].xPos, itemSmallClock[smallmIndex-1].yPos);

    if(sIndex <= 30)
      smallsIndex = sIndex + 30;
    else
      smallsIndex = sIndex - 30;
    p.setPen(QPen(QColor(255, 0, 0), 2));
    p.drawLine(xCenter, yCenter, itemSmallClock[smallsIndex-1].xPos, itemSmallClock[smallsIndex-1].yPos);

    QRect rects, recth;
    recth.setX(xCenter - 5);
    recth.setY(yCenter - 5);
    recth.setWidth(10);
    recth.setHeight(10);

    rects.setX(xCenter - 3);
    rects.setY(yCenter - 3);
    rects.setWidth(6);
    rects.setHeight(6);
    p.setPen(QPen(QColor(0, 0, 0), 1));
    p.setBrush(QBrush(QColor(0, 0, 0)));
    p.drawEllipse(recth);
    p.setPen(QPen(QColor(255, 0, 0), 1));
    p.setBrush(QBrush(QColor(255, 0, 0)));
    p.drawEllipse(rects);
    m_pView->UpdateTimeView();
  }
  p.end();
}

void DoraClockView::timerEvent(QTimerEvent *e)
{
  timID = e->timerId();
  killedTimer = false;
  updateView();
}

void DoraClockView::updateView()
{
  DoraCentralDocument *pDoc = GetDocument();
  int h = pDoc->curentHour;
  int m = pDoc->curentMin;
  int s = pDoc->curentSec;
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralView *m_pCentralView = m_pMainFrame->GetCentralView();
  repaint(false);  
  if((h == 0) && (m == 0) && (s == 0)) {
    m_pCentralView->setCurentDate();
  }
}

void DoraClockView::stopTimer()
{
  if(!killedTimer) {
    killTimer(timID);
    killedTimer = true;
  }
  updateView();
}
