#include "DoraCentralDocument.h"
#include "DoraCentralView.h"
#include "DoraMainFrame.h"
#include "DoraComboBox.h"

DoraRuntimeClass DoraComboBox::classDoraComboBox={"DoraComboBox", DoraComboBox::CreateView, NULL};

DoraClassObject *DoraComboBox::CreateView(QWidget *parent)
{
  return new DoraComboBox(parent);
}

DoraClassObject *DoraComboBox::CreateDocument()
{
  return NULL;
}

void DoraComboBox::OnDraw(QPainter *p)
{
  p->setFont(font);
  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->setBrush(QBrush(QColor(192, 192, 192)));
  p->drawRect(clientRect);

  m_pStaticCtrl->show();
  m_pScrBarButton->show();
}

void DoraComboBox::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);

  QFontMetrics fm(editFont);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int w = sizeFont.width();
  int h = fm.height() - fm.descent();
  
  int xEdit = 0;
  int yEdit = 0;
  int wEdit = (maxLen + 1) * w;
  int hEdit = h + 4;

  QRect editRect;
  editRect.setX(xEdit);
  editRect.setY(yEdit);
  editRect.setWidth(wEdit);
  editRect.setHeight(hEdit);
  m_pStaticCtrl->setGeometry(editRect);
  m_pStaticCtrl->hide();
  
  int xButton = (clientRect.x() + editRect.width());
  int yButton = 0;
  int wButton = h + 4;
  int hButton = h + 4;
  
  QRect buttonRect;
  buttonRect.setX(xButton);
  buttonRect.setY(yButton);
  buttonRect.setWidth(wButton);
  buttonRect.setHeight(hButton);

  m_pScrBarButton->setGeometry(buttonRect);
  m_pScrBarButton->hide();
  
  
}

void DoraComboBox::OnCreate(DoraClassObject *pObject, DoraBaseView *pView)
{
  m_pObject = pObject;
  m_pView   = pView;
  m_pDialog = NULL;
  setFont(font);
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = GetDocument();
  editFont = QFont(pDoc->getDefaultFont(5));

  DoraRuntimeClass *runtimeStaticCtrl = RUNTIME_CLASS(DoraStaticCtrl);
  DoraClassObject *objectStaticCtrl = runtimeStaticCtrl->CreateView(this);
  m_pStaticCtrl = dynamic_cast<DoraStaticCtrl *>(objectStaticCtrl);
  m_pStaticCtrl->convChar = pDoc->convChar;
  m_pStaticCtrl->selectNewFont(pDoc->getDefaultFont(5));
  m_pStaticCtrl->OnCreate(m_pObject, this);
  Str35 caption;
  caption = "";
  if((nID == ID_COMBOBOX_CALENDAR) && (nScrollID == ID_SCROLLBOX_CALENDAR))
    caption = pDoc->itemMonthName[pDoc->curentMonth - 1].name;
  if((nID == ID_COMBOBOX_MENU) && (nScrollID == ID_SCROLLBOX_ENCODING))
    caption = pDoc->itemEncodingName[pDoc->curentEncoding - 1].name;
  if((nID == ID_COMBOBOX_MENU) && (nScrollID == ID_SCROLLBOX_LANG))
    caption = pDoc->itemLangName[pDoc->curentLang - 1].name;
  m_pStaticCtrl->setCaption(caption);
  m_pStaticCtrl->setColor(QColor(0, 0, 0), QColor(0, 0, 0), QColor(255, 255, 255));
  m_pStaticCtrl->hide();

  DoraRuntimeClass *runtimeScrBarButton = RUNTIME_CLASS(DoraScrBarButton);
  DoraClassObject *objectScrBarButton = runtimeScrBarButton->CreateView(this);
  m_pScrBarButton = dynamic_cast<DoraScrBarButton *>(objectScrBarButton);
  m_pScrBarButton->convChar = pDoc->convChar;
  m_pScrBarButton->selectNewFont(pDoc->getDefaultFont(5));
  m_pScrBarButton->OnCreate(m_pObject, this, ID_SCROLLBARBUTTON_DOWN);
  m_pScrBarButton->hide();
  
}

void DoraComboBox::OnCreate(DoraClassObject *pObject, DoraBaseDialog *pDialog)
{
  m_pObject = pObject;
  m_pView   = NULL;
  m_pDialog = pDialog;
  setFont(font);
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = GetDocument();
  editFont = QFont(pDoc->getDefaultFont(5));

  DoraRuntimeClass *runtimeStaticCtrl = RUNTIME_CLASS(DoraStaticCtrl);
  DoraClassObject *objectStaticCtrl = runtimeStaticCtrl->CreateView(this);
  m_pStaticCtrl = dynamic_cast<DoraStaticCtrl *>(objectStaticCtrl);
  m_pStaticCtrl->convChar = pDoc->convChar;
  m_pStaticCtrl->selectNewFont(pDoc->getDefaultFont(5));
  m_pStaticCtrl->OnCreate(m_pObject, this);
  Str35 caption;
  caption = "";
  if((nID == ID_COMBOBOX_CALENDAR) && (nScrollID == ID_SCROLLBOX_CALENDAR))
    caption = pDoc->itemMonthName[pDoc->curentMonth - 1].name;
  if((nID == ID_COMBOBOX_DIALOG) && (nScrollID == ID_SCROLLBOX_ENCODING))
    caption = pDoc->itemEncodingName[pDoc->curentEncoding - 1].name;
  if((nID == ID_COMBOBOX_DIALOG) && (nScrollID == ID_SCROLLBOX_LANG))
    caption = pDoc->itemLangName[pDoc->curentLang - 1].name;
  m_pStaticCtrl->setCaption(caption);
  m_pStaticCtrl->setColor(QColor(0, 0, 0), QColor(0, 0, 0), QColor(255, 255, 255));
  m_pStaticCtrl->hide();

  DoraRuntimeClass *runtimeScrBarButton = RUNTIME_CLASS(DoraScrBarButton);
  DoraClassObject *objectScrBarButton = runtimeScrBarButton->CreateView(this);
  m_pScrBarButton = dynamic_cast<DoraScrBarButton *>(objectScrBarButton);
  m_pScrBarButton->convChar = pDoc->convChar;
  m_pScrBarButton->selectNewFont(pDoc->getDefaultFont(5));
  m_pScrBarButton->OnCreate(m_pObject, this, ID_SCROLLBARBUTTON_DOWN);
  m_pScrBarButton->hide();
}

void DoraComboBox::addItem(Str35 strName, const int index = 1)
{
  m_pScrollView->addExpand(strName, index);
}

void DoraComboBox::setBoxRect(QRect rect)
{
  boxRect.setX(rect.x());
  boxRect.setY(rect.y());
  boxRect.setWidth(rect.width());
  boxRect.setHeight(rect.height());
}

void DoraComboBox::PostMouseMessage(int ctrl, int index)
{
  DoraCentralDocument *pDoc = GetDocument();

  if(ctrl == POST_SCROLLBARBUTTON_DOWN) {
    if(isExpand) {
      hideScrollView();
    } else {
      showScrollView();
    }
    isExpand = !isExpand;
  }
  if(ctrl == POST_SCROLLBOX) {
    setDefault(index);
    if(isExpand)
      m_pScrollView->hide();
    isExpand = FALSE;
    int ctrlID = nScrollID;
    if(m_pView != NULL)
      m_pView->PostMouseMessage(ctrlID, index);
    if(m_pDialog != NULL)
      m_pDialog->PostMouseMessage(ctrlID, index);
    hideScrollView();
  }
}

void DoraComboBox::setDefault(const int index = 1)
{ 
  DoraCentralDocument *pDoc = GetDocument();

  Str35 caption;
  caption = "";
  if((nID == ID_COMBOBOX_CALENDAR) && (nScrollID == ID_SCROLLBOX_CALENDAR))
    caption = pDoc->itemMonthName[index - 1].name;
  if((nID == ID_COMBOBOX_MENU) && (nScrollID == ID_SCROLLBOX_ENCODING))
    caption = pDoc->itemEncodingName[index - 1].name;
  if((nID == ID_COMBOBOX_MENU) && (nScrollID == ID_SCROLLBOX_LANG))
    caption = pDoc->itemLangName[index - 1].name;
  if((nID == ID_COMBOBOX_DIALOG) && (nScrollID == ID_SCROLLBOX_ENCODING))
    caption = pDoc->itemEncodingName[index - 1].name;
  if((nID == ID_COMBOBOX_DIALOG) && (nScrollID == ID_SCROLLBOX_LANG))
    caption = pDoc->itemLangName[index - 1].name;
  m_pStaticCtrl->setCaption(caption);
  m_pStaticCtrl->repaint(false);
  if(m_pScrollView != NULL)
    m_pScrollView->m_pScrollBox->setCheckElement(index);
}

DoraCentralDocument *DoraComboBox::GetDocument()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = m_pMainFrame->GetDocument();
  return pDoc;
}

CString DoraComboBox::getValue()
{
  CString value;
  value = "";value = m_pStaticCtrl->getValue();
  return value;
}

void DoraComboBox::DestroyView()
{
}

void DoraComboBox::showScrollView()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = GetDocument();
  if(m_pScrollView != NULL) {
    m_pScrollView->hide();
    delete m_pScrollView;
    m_pScrollView = NULL;
  }
  DoraRuntimeClass *runtimeScrollView = RUNTIME_CLASS(DoraScrollView);  
  DoraClassObject *objectScrollView;
  if(m_pView != NULL) {
    if(nID == ID_COMBOBOX_MENU)
      objectScrollView = runtimeScrollView->CreateView(m_pMainFrame);
    else
      objectScrollView = runtimeScrollView->CreateView(m_pView);
  }
  if(m_pDialog != NULL) {
    if(nID == ID_COMBOBOX_MENU)
      objectScrollView = runtimeScrollView->CreateView(m_pMainFrame);
    else
      objectScrollView = runtimeScrollView->CreateView(m_pDialog);
  }
  m_pScrollView = dynamic_cast<DoraScrollView *>(objectScrollView);
  m_pScrollView->convChar = pDoc->convChar;
  m_pScrollView->selectNewFont(pDoc->getDefaultFont(5));
  m_pScrollView->okScrollBar = okScrollBar;
  m_pScrollView->maxRow = maxRow;
  m_pScrollView->nID = nScrollID;
  m_pScrollView->convChar = convChar;
  m_pScrollView->OnCreate(m_pObject, this);
  if(m_pView != NULL)
    m_pView->PostMouseMessage(nID, nScrollID);
  if(m_pDialog != NULL)
    m_pDialog->PostMouseMessage(nID, nScrollID);
  m_pScrollView->m_pScrollBox->maxRow = maxRow;
  m_pScrollView->setGeometry(boxRect);
  m_pScrollView->show();
}

void DoraComboBox::hideScrollView()
{
  if(m_pScrollView != NULL) {
    m_pScrollView->hide();
    delete m_pScrollView;
    m_pScrollView = NULL;
  }
}

void DoraComboBox::UpdateConvChar()
{
  m_pStaticCtrl->convChar = convChar;
  m_pStaticCtrl->repaint(false);
}
