#include "DoraEditBox.h"

DoraRuntimeClass DoraEditBox::classDoraEditBox={"DoraEditBox", DoraEditBox::CreateView, NULL};

DoraClassObject *DoraEditBox::CreateView(QWidget *parent)
{
  return new DoraEditBox(parent);
}

DoraClassObject *DoraEditBox::CreateDocument()
{
  return NULL;
}

void DoraEditBox::OnCreate(DoraClassObject *pObject, DoraBaseView *pView)
{
  m_pObject = pObject;
  m_pDialog = NULL;
  m_pView   = pView;
}

void DoraEditBox::OnCreate(DoraClassObject *pObject, DoraBaseDialog *pDialog)
{
  m_pObject = pObject;
  m_pDialog = pDialog;
  m_pView   = NULL;
}

void DoraEditBox::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);

  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int wf = sizeFont.width();
  int hf = fm.height() - fm.descent();
  int maxElement = editItem.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    int x = editItem[i].xPos * wf;
    int y = (editItem[i].yPos * (hf + 6)) + 2;
    int w = editItem[i].len * wf;
    int h = hf + 6;
    QRect rect;
    rect.setX(x);
    rect.setY(y);
    rect.setWidth(w);
    rect.setHeight(h);
    editItem[i].editString->setGeometry(rect);
    editItem[i].editString->hide();
  }
}

void DoraEditBox::OnDraw(QPainter *p)
{
  p->setFont(font);
  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->setBrush(QBrush(QColor(192, 192, 192)));
  p->drawRect(clientRect);

  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int wf = sizeFont.width();
  int hf = fm.height() - fm.descent();
  int maxElement = editItem.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    int x = editItem[i].xPos * wf;
    int y = (((editItem[i].yPos - 1) * (hf + 6)) + 2) + hf;
    CString cap;
    cap = "";cap = editItem[i].caption;
    int len = strlen(cap.p);
    for(int j = 0; j <= len - 1; j++) {
      char *s = new char[2];
      s[0] = '\0';
      sprintf(s, "%c\0", cap.p[j]);
      QString qStr;
      qStr = "";qStr = convChar.getCharCodec(s);
      p->setPen(QPen(QColor(0, 0, 0), 1));
      p->setBrush(QBrush(QColor(192, 192, 192)));
      p->drawText(x, y, qStr);
      x += wf;
      delete s;
    }
    editItem[i].editString->show();
  }
}

void DoraEditBox::setDialogAttr(const int index)
{
  int maxElement = editItem.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    DoraEditString *editString;
    editString = editItem[i].editString;
    editString->hide();
    delete editString;
    editString = NULL;
  }
  editItem.removeAllElement();
  attr.setAttrDialog(index);
  if(index == 2)
    attr.setAttrBDate(prefix);
  maxElement = attr.getMaxItem();
  for(int i = 0; i <= maxElement - 1; i++) {
    strItemEdit item;
    DoraRuntimeClass *runtimeEditString = RUNTIME_CLASS(DoraEditString);
    DoraClassObject *objectEditString = runtimeEditString->CreateView(this);
    item.editString = dynamic_cast<DoraEditString *>(objectEditString);
    item.editString->lenStr = attr.itemAttr[i].len;
    item.editString->legal = "";item.editString->legal = attr.itemAttr[i].legal;
    item.editString->status = attr.itemAttr[i].status;
    item.editString->selectNewFont(font);
    item.editString->convChar = convChar;
    item.editString->OnCreate(m_pObject, this, i + 1);
    item.editString->hide();
    item.caption = "";item.caption = attr.itemAttr[i].caption;
    item.xPos = attr.itemAttr[i].xPos;
    item.yPos = attr.itemAttr[i].yPos;
    item.len = attr.itemAttr[i].len + 1;
    editItem.setElement(item, i + 1);
    nID = attr.itemAttr[i].nID;
  }
  int newIndex = getNextNotReadonly(1);
  editItem[newIndex - 1].editString->setFocus();
}

void DoraEditBox::PostEditMessage(const int indexEdit)
{
  editItem[indexEdit - 1].editString->setFocus();
}

void DoraEditBox::PostKeyPressMessage(int index, int keyCode)
{
  int maxElement = editItem.maxElement();
  int i = index;
  int newIndex;

  switch(keyCode) {
  case Key_Up :
    if(i <= 1)
      i = 2;
    newIndex = getPrevNotReadonly(i - 1);
    i = newIndex;
    break;
  case Key_Down :
    if(i >= maxElement)
      i = maxElement - 1;
    newIndex = getNextNotReadonly(i + 1);
    i = newIndex;
    break;
  case Key_Tab :
    if(i <= maxElement)
      i++;
    newIndex = getNextNotReadonly(i);
    i = newIndex;
    break;
  case Key_Return :
  case Key_Enter :
    if(i <= maxElement)
      i++;
    newIndex = getNextNotReadonly(i);
    i = newIndex;
    if(i > maxElement) {
      if(m_pDialog != NULL) {
	m_pDialog->PostKeyPressMessage(index, keyCode);
	i = 1;
      }
    }
    break;
  case Key_F1       :
  case Key_F2       :
  case Key_F3       :
  case Key_F4       :
  case Key_PageUp   :
  case Key_PageDown :
    if(m_pDialog != NULL) {
      m_pDialog->PostKeyPressMessage(index, keyCode);
      i = 1;
    }
    break;
  }
  editItem[i - 1].editString->setFocus();
}

void DoraEditBox::PostKeyStatMessage(int stat, int keyCode)
{
  if(stat == ControlButton) {
    if(m_pView != NULL)
      m_pView->PostKeyStatMessage(stat, keyCode);
    if(m_pDialog != NULL)
      m_pDialog->PostKeyStatMessage(stat, keyCode);
  }
}

void DoraEditBox::setData(DataRecord data, const int focusIndex)
{
  int maxElement = editItem.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    CString value;
    value = "";value = data.Pole[i];
    editItem[i].editString->setEditString(value);
    editItem[i].editString->repaint(false);
  }
  if(editItem[focusIndex-1].editString->isFocus) {
    int newIndex = getNextNotReadonly(focusIndex);
    editItem[newIndex-1].editString->setOnFocus();
  } else {
    int newIndex = getNextNotReadonly(focusIndex);
    editItem[newIndex-1].editString->setFocus();
  }
}

void DoraEditBox::getData(DataRecord *data)
{
  int maxElement = editItem.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    data->Pole[i] = "";
    CString s;
    s = "";s = editItem[i].editString->getEditString();
    data->Pole[i] = s.p;
  }
}

Str80 DoraEditBox::getString(const int index)
{
  Str80 str;
  CString s;
  s = "";s = editItem[index-1].editString->getEditString();
  str = "";str = s.p;
  return str;
}

void DoraEditBox::setString(const int index, const Str80 str)
{
  int maxElement = editItem.maxElement();
  CString value;
  value = "";value = str.p;
  if((index >= 1) && (index <= maxElement)) {
    editItem[index - 1].editString->setEditString(value);
    editItem[index - 1].editString->repaint(false);
  }
}


int DoraEditBox::getNextNotReadonly(const int index)
{
  int newIndex = index;
  int maxElement = editItem.maxElement();
  int i;
  for(i = index - 1; i <= maxElement - 1; i++) {
    if(editItem[i].editString->status == ID_STATUS_NOTREADONLY) {
      newIndex = i + 1;
      break;
    }
  }
  if((i == maxElement - 1) && (editItem[i].editString->status == ID_STATUS_READONLY))
    newIndex = index - 1;
  return newIndex;
}

int DoraEditBox::getPrevNotReadonly(const int index)
{
  int newIndex = index;
  int i = index - 1;
  do {
    if(editItem[i].editString->status == ID_STATUS_NOTREADONLY) {
      newIndex = i + 1;
      break;
    }
    i--;
  } while(i >= 1);
  if(i <= 0) 
    i = 0;
  if((i == 0) && (editItem[i].editString->status == ID_STATUS_READONLY))
    newIndex = index + 1;
  return newIndex;
}

void DoraEditBox::UpdateConvChar()
{
  int maxElement = editItem.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    strItemEdit item;
    item = editItem[i];
    item.editString->convChar = convChar;
    item.editString->repaint(false);
    editItem.setElement(item, i + 1);
  }
}
