#include "DoraCentralDocument.h"
#include "DoraMainFrame.h"
#include "DoraCentralView.h"
#include "DoraGlobMenuView.h"

DoraRuntimeClass DoraGlobMenuView::classDoraGlobMenuView={"DoraGlobMenuView", DoraGlobMenuView::CreateView, NULL};

DoraClassObject *DoraGlobMenuView::CreateView(QWidget *parent)
{
  return new DoraGlobMenuView(parent);
}

DoraClassObject *DoraGlobMenuView::CreateDocument()
{
  return NULL;
}

void DoraGlobMenuView::OnDraw(QPainter *p)
{
  p->setFont(font);
  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->setBrush(QBrush(QColor(192, 192, 192)));
  p->drawRect(clientRect);

  int x1 = clientRect.x() + 1;
  int y1 = clientRect.y() + 1;
  int x2 = clientRect.width() - 2;
  int y2 = clientRect.height() - 2;

  p->setPen(QPen(QColor(255, 255, 255), 1));
  p->drawLine(x1, y2, x1, y1);
  p->drawLine(x1, y1, x2, y1);
  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->drawLine(x1, y2, x2, y2);
  p->drawLine(x2, y2, x2, y1);

  int maxElement = itemMenu.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    itemMenu[i].itemButton->show();
  }
  m_pComboBoxE->show();
  m_pComboBoxL->show();
}

void DoraGlobMenuView::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);

  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int wf = sizeFont.width();
  int hf = fm.height() - fm.descent();
  int x = clientRect.x() + wf;
  int y = clientRect.y() + 2;
  int h = clientRect.height() - 4;
  int maxElement = itemMenu.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    int len = strlen(itemMenu[i].itemButton->caption.p);
    int w = wf * (len + 2);
    QRect rect;
    rect.setX(x);
    rect.setY(y);
    rect.setWidth(w);
    rect.setHeight(h);
    strItem item;
    item = itemMenu[i];
    item.xPos = x;
    itemMenu.setElement(item, i + 1);
    itemMenu[i].itemButton->setGeometry(rect);
    itemMenu[i].itemButton->hide();
    x += (w + 1);
  }
  QFontMetrics fmCombo(comboFont);
  QSize sizeFontCombo = fmCombo.size(SingleLine, "H", 1);
  int wfc = sizeFont.width();
  int hfc = fmCombo.height() - fmCombo.descent();
  int xCombo = x + 2;
  int yCombo = y + 2;
  int hCombo = (fmCombo.height() - fmCombo.descent()) + 4;
  int wCombo = (15 * wfc) + (5 * wfc) + 4;
  QRect comboRectE;
  comboRectE.setX(xCombo);
  comboRectE.setY(yCombo);
  comboRectE.setWidth(wCombo);
  comboRectE.setHeight(hCombo);
  m_pComboBoxE->setGeometry(comboRectE);
  int xBoxComboE = comboRectE.x();
  int yBoxComboE = comboRectE.y() + comboRectE.height() + 1;
  int wBoxComboE = comboRectE.width();
  int hBoxComboE = hCombo * 12;
  QRect boxRectE;
  boxRectE.setX(xBoxComboE);
  boxRectE.setY(yBoxComboE);
  boxRectE.setWidth(wBoxComboE);
  boxRectE.setHeight(hBoxComboE - 6);
  m_pComboBoxE->setBoxRect(boxRectE);
  m_pComboBoxE->hide();

  xCombo += wCombo + 2;
  QRect comboRectL;
  comboRectL.setX(xCombo);
  comboRectL.setY(yCombo);
  comboRectL.setWidth(wCombo);
  comboRectL.setHeight(hCombo);
  m_pComboBoxL->setGeometry(comboRectL);
  int xBoxComboL = comboRectL.x();
  int yBoxComboL = comboRectL.y() + comboRectL.height() + 1;
  int wBoxComboL = comboRectL.width();
  int hBoxComboL = hCombo * 4;
  QRect boxRectL;
  boxRectL.setX(xBoxComboL);
  boxRectL.setY(yBoxComboL);
  boxRectL.setWidth(wBoxComboL);
  boxRectL.setHeight(hBoxComboL - 2);
  m_pComboBoxL->setBoxRect(boxRectL);
  m_pComboBoxL->hide();
}

void DoraGlobMenuView::OnCreate(DoraClassObject *pObject)
{
  m_pObject = pObject;
  setFont(font);

  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = GetDocument();
  comboFont = pDoc->getDefaultFont(5);

  DoraRuntimeClass *runtimeComboBoxE = RUNTIME_CLASS(DoraComboBox);
  DoraClassObject *objectComboBoxE = runtimeComboBoxE->CreateView(this);
  m_pComboBoxE = dynamic_cast<DoraComboBox *>(objectComboBoxE);
  m_pComboBoxE->selectNewFont(pDoc->getDefaultFont(5));
  m_pComboBoxE->convChar = convChar;
  m_pComboBoxE->nID = ID_COMBOBOX_MENU;
  m_pComboBoxE->maxLen = 15;
  m_pComboBoxE->okScrollBar = true;
  m_pComboBoxE->maxRow = 12;
  m_pComboBoxE->nScrollID = ID_SCROLLBOX_ENCODING;
  m_pComboBoxE->OnCreate(m_pObject, this);
  m_pComboBoxE->hide();

  DoraRuntimeClass *runtimeComboBoxL = RUNTIME_CLASS(DoraComboBox);
  DoraClassObject *objectComboBoxL = runtimeComboBoxL->CreateView(this);
  m_pComboBoxL = dynamic_cast<DoraComboBox *>(objectComboBoxL);
  m_pComboBoxL->selectNewFont(pDoc->getDefaultFont(5));
  m_pComboBoxL->convChar = convChar;
  m_pComboBoxL->nID = ID_COMBOBOX_MENU;
  m_pComboBoxL->maxLen = 15;
  m_pComboBoxL->okScrollBar = true;
  m_pComboBoxL->maxRow = 4;
  m_pComboBoxL->nScrollID = ID_SCROLLBOX_LANG;
  m_pComboBoxL->OnCreate(m_pObject, this);
  m_pComboBoxL->hide();
}

DoraCentralDocument *DoraGlobMenuView::GetDocument()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = m_pMainFrame->GetDocument();
  return pDoc;
}

void DoraGlobMenuView::setGlobalMenu()
{
  DoraCentralDocument *pDoc = GetDocument();
  Str35 prefix;
  prefix = pDoc->itemLangName[pDoc->curentLang - 1].prefix;
  itemMenu.removeAllElement();
  attr.setGlobalMenu(prefix);

  int maxElement = attr.itemGlobMenu.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    strItem item;
    DoraRuntimeClass *runtimeMenuButton = RUNTIME_CLASS(DoraMenuButton);
    DoraClassObject *objectMenuButton = runtimeMenuButton->CreateView(this);
    item.itemButton = dynamic_cast<DoraMenuButton *>(objectMenuButton);
    item.itemButton->selectNewFont(font);
    item.itemButton->convChar = convChar;
    item.itemButton->OnCreate(m_pObject, this);
    item.itemButton->hide();
    item.nID = attr.itemGlobMenu[i].nID;
    item.itemButton->setCaption(attr.itemGlobMenu[i].strMenu);
    item.itemButton->nID = i + 1;
    item.itemButton->statusID = attr.itemGlobMenu[i].itemType;
    item.itemButton->ctrlID = ID_SIGNAL_MENUBUTTON;
    itemMenu.setElement(item, i + 1);
  }
}

void DoraGlobMenuView::setPopupMenu(const int index)
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralView *m_pCentralView = m_pMainFrame->GetCentralView();
  DoraCentralDocument *pDoc = m_pMainFrame->GetDocument();
  Str35 prefix;
  prefix = "";prefix = pDoc->itemLangName[pDoc->curentLang - 1].prefix;
  if(m_pPopupMenu != NULL) {
    m_pPopupMenu->hide();
    delete m_pPopupMenu;
    m_pPopupMenu = NULL;
  }
  DoraRuntimeClass *runtimePopupMenu = RUNTIME_CLASS(DoraPopupMenu);
  DoraClassObject *objectPopupMenu = runtimePopupMenu->CreateView(m_pMainFrame);
  m_pPopupMenu = dynamic_cast<DoraPopupMenu *>(objectPopupMenu);
  m_pPopupMenu->selectNewFont(font);
  m_pPopupMenu->convChar = convChar;
  m_pPopupMenu->OnCreate(m_pObject, this);
  m_pPopupMenu->setPopupItem(itemMenu[index-1].nID);
  m_pPopupMenu->hide();

  attr.setPopupMenu(itemMenu[index - 1].nID, prefix);

  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int wf = sizeFont.width();
  int hf = fm.height() - fm.descent();
  int h = ((hf + 2) * attr.getMaxEPopupItem()) + 6;
  int w = (wf * attr.getMaxLPopupItem()) + attr.getMaxLPopupItem() + 4;
  int x = itemMenu[index - 1].xPos;
  int y = (m_pMainFrame->yGMenu + m_pMainFrame->hGMenu) - 2;

  QRect popupRect;
  popupRect.setX(x);
  popupRect.setY(y);
  popupRect.setWidth(w);
  popupRect.setHeight(h);
  m_pPopupMenu->setGeometry(popupRect);
  m_pPopupMenu->show();
  
}

void DoraGlobMenuView::PostMouseMove(int index)
{
  itemMenu[index-1].itemButton->setFocus();
  if((isExpand) && (oldIndex != index))
    setPopupMenu(index);
  if(isExpand) {
    itemMenu[index-1].itemButton->setStatus(ID_STATUS_BUTTON_PUSHED);
    itemMenu[index-1].itemButton->repaint(false);
  }
  oldIndex = index;
}

void DoraGlobMenuView::OnFocus()
{
  if(m_pPopupMenu != NULL) {
    m_pPopupMenu->hide();
    delete m_pPopupMenu;
    m_pPopupMenu = NULL;
  }
  isExpand = false;
  int maxElement = itemMenu.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    itemMenu[i].itemButton->setStatus(ID_STATUS_BUTTON_NORMAL);
    itemMenu[i].itemButton->repaint(false);
  }
}

void DoraGlobMenuView::PostMouseMessage(int ctrl, int index)
{  
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = GetDocument();

  if(ctrl == ID_SIGNAL_MENUBUTTON) {
    setPopupMenu(index);
    isExpand = true;
    int maxElement = itemMenu.maxElement();
    for(int i = 0; i <= maxElement - 1; i++) {
      if(i != index - 1)
	itemMenu[i].itemButton->setStatus(ID_STATUS_BUTTON_NORMAL);
      itemMenu[i].itemButton->repaint(false);
    }
  }
  if(ctrl ==  ID_SIGNAL_POPUPMENU) {
    isExpand = false;
    int maxElement = itemMenu.maxElement();
    for(int i = 0; i <= maxElement - 1; i++) {
      if(i != index - 1)
	itemMenu[i].itemButton->setStatus(ID_STATUS_BUTTON_NORMAL);
      itemMenu[i].itemButton->repaint(false);
    }
    if(m_pPopupMenu != NULL) {
      m_pPopupMenu->hide();
      delete m_pPopupMenu;
      m_pPopupMenu = NULL;
    }
    if(index == 101)
      m_pMainFrame->OnReklDialog();
    if(index == 201)
      m_pMainFrame->OnBioritms();
    if(index == 301)
      m_pMainFrame->OnAbautView();
    if((index == 401) || (index == 103))
      m_pMainFrame->OnCloseView();
  }
  if(ctrl == ID_COMBOBOX_MENU) {
    if(index == ID_SCROLLBOX_ENCODING) {
      int maxElement = pDoc->itemEncodingName.maxElement();
      for(int i = 0; i <= maxElement - 1; i++) {
	m_pComboBoxE->addItem(pDoc->itemEncodingName[i].name, i + 1);
      }
      m_pComboBoxE->setDefault(pDoc->curentEncoding);
      m_pComboBoxE->m_pScrollView->m_pScrollBox->setRectElement(pDoc->curentEncoding);
    }
    if(index == ID_SCROLLBOX_LANG) {
      int maxElement = pDoc->itemLangName.maxElement();
      for(int i = 0; i <= maxElement - 1; i++) {
	m_pComboBoxL->addItem(pDoc->itemLangName[i].name, i + 1);
      }
      m_pComboBoxL->setDefault(pDoc->curentLang);
      m_pComboBoxL->m_pScrollView->m_pScrollBox->setRectElement(pDoc->curentLang);
    }
  }
  if(ctrl == ID_SCROLLBOX_ENCODING) {
    Str35 encodingName;
    encodingName = "";
    encodingName = pDoc->itemEncodingName[index-1].name;
    pDoc->setNewConvChar(encodingName);
    convChar = pDoc->convChar;
    pDoc->curentEncoding = index;
    int maxElement = itemMenu.maxElement();
    for(int i = 0; i <= maxElement - 1; i++) {
      strItem item;
      item = itemMenu[i];
      item.itemButton->convChar = convChar;
      item.itemButton->repaint(false);
      itemMenu.setElement(item, i + 1);
    }
    m_pComboBoxE->convChar = convChar;
    m_pComboBoxE->UpdateConvChar();
    m_pComboBoxL->convChar = convChar;
    m_pComboBoxL->UpdateConvChar();
    m_pMainFrame->UpdateConvChar();
  }
  if(ctrl == ID_SCROLLBOX_LANG) {
    int maxLang = pDoc->itemLangName.maxElement();
    Str35 langName, prefix;
    langName = "";prefix = "";
    if(index <= 0) {
      langName = pDoc->itemLangName[0].name;
      prefix = pDoc->itemLangName[0].prefix;
    }
    if(index >= maxLang) {
      langName = pDoc->itemLangName[maxLang - 1].name;
      prefix = pDoc->itemLangName[maxLang - 1].prefix;
    }
    if((index >= 1) && (index <= maxLang)) {
      langName = pDoc->itemLangName[index - 1].name;
      prefix = pDoc->itemLangName[index - 1].prefix;
    }
    pDoc->setNewLang(langName);
    convChar = pDoc->convChar;
    pDoc->loadLangFile(prefix);
    int maxElement = itemMenu.maxElement();
    for(int i = 0; i <= maxElement - 1; i++) {
      strItem item;
      item = itemMenu[i];
      item.itemButton->convChar = convChar;
      item.itemButton->repaint(false);
      itemMenu.setElement(item, i + 1);
    }
    setNewLang(prefix);
    m_pComboBoxE->convChar = convChar;
    m_pComboBoxE->UpdateConvChar();
    m_pComboBoxE->setDefault(pDoc->curentEncoding);
    m_pComboBoxL->convChar = convChar;
    m_pComboBoxL->UpdateConvChar();
    m_pComboBoxL->setDefault(pDoc->curentLang);
    m_pMainFrame->UpdateConvChar();
  }
}

void DoraGlobMenuView::MouseMove()
{
  setFocus();
}

void DoraGlobMenuView::PostMouseClick(int index)
{
  if(m_pPopupMenu != NULL) {
    m_pPopupMenu->hide();
    delete m_pPopupMenu;
    m_pPopupMenu = NULL;
  }
  isExpand = false;
  int maxElement = itemMenu.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    itemMenu[i].itemButton->setStatus(ID_STATUS_BUTTON_NORMAL);
    itemMenu[i].itemButton->repaint(false);
  }  
}

void DoraGlobMenuView::setNewLang(Str35 langPrefix)
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = m_pMainFrame->GetDocument();
  Str35 fileName;
  fileName = "";fileName = "GLOBMENU";
  pDoc->loadConfigDir(langPrefix, fileName);
  int maxElement = pDoc->itemStrLangCaption.maxElement();
  if(maxElement >= 1) {
    QFontMetrics fm(font);
    QSize sizeFont = fm.size(SingleLine, "H", 1);
    int wf = sizeFont.width();
    int hf = fm.height() - fm.descent();
    int x = clientRect.x() + wf;
    int y = clientRect.y() + 2;
    int h = clientRect.height() - 4;
    for(int i = 0; i <= maxElement - 1; i++) {
      strItem item;
      Str35 caption;
      caption = "";caption = pDoc->itemStrLangCaption[i].str;
      int len = strlen(caption.p);
      int w = wf * (len + 2);
      QRect rect;
      rect.setX(x);
      rect.setY(y);
      rect.setWidth(w);
      rect.setHeight(h);
      item = itemMenu[i];
      item.itemButton->setCaption(caption);
      item.itemButton->setGeometry(rect);
      item.itemButton->repaint(false);
      item.xPos = x;
      itemMenu.setElement(item, i+ 1);
      x += (w + 1);
    }
    QFontMetrics fmCombo(comboFont);
    QSize sizeFontCombo = fmCombo.size(SingleLine, "H", 1);
    int wfc = sizeFont.width();
    int hfc = fmCombo.height() - fmCombo.descent();
    int xCombo = x + 2;
    int yCombo = y + 2;
    int hCombo = (fmCombo.height() - fmCombo.descent()) + 4;
    int wCombo = (15 * wfc) + (5 * wfc) + 7;
    QRect comboRectE;
    comboRectE.setX(xCombo);
    comboRectE.setY(yCombo);
    comboRectE.setWidth(wCombo);
    comboRectE.setHeight(hCombo);
    m_pComboBoxE->setGeometry(comboRectE);
    int xBoxComboE = comboRectE.x();
    int yBoxComboE = comboRectE.y() + comboRectE.height() + 1;
    int wBoxComboE = comboRectE.width();
    int hBoxComboE = hCombo * 12;
    QRect boxRectE;
    boxRectE.setX(xBoxComboE);
    boxRectE.setY(yBoxComboE);
    boxRectE.setWidth(wBoxComboE);
    boxRectE.setHeight(hBoxComboE - 6);
    m_pComboBoxE->setBoxRect(boxRectE);
    m_pComboBoxE->repaint(false);

    xCombo += wCombo + 2;
    QRect comboRectL;
    comboRectL.setX(xCombo);
    comboRectL.setY(yCombo);
    comboRectL.setWidth(wCombo);
    comboRectL.setHeight(hCombo);
    m_pComboBoxL->setGeometry(comboRectL);
    int xBoxComboL = comboRectL.x();
    int yBoxComboL = comboRectL.y() + comboRectL.height() + 1;
    int wBoxComboL = comboRectL.width();
    int hBoxComboL = hCombo * 4;
    QRect boxRectL;
    boxRectL.setX(xBoxComboL);
    boxRectL.setY(yBoxComboL);
    boxRectL.setWidth(wBoxComboL);
    boxRectL.setHeight(hBoxComboL - 2);
    m_pComboBoxL->setBoxRect(boxRectL);
    m_pComboBoxL->repaint(false);
  }
  pDoc->addMonthName();
  pDoc->addDayName();
}
