#include "DoraCentralDocument.h"
#include "DoraCentralView.h"
#include "DoraMainFrame.h"
#include "DoraInfoGroup.h"

DoraRuntimeClass DoraInfoGroup::classDoraInfoGroup={"DoraInfoGroup", DoraInfoGroup::CreateView, NULL};

DoraClassObject *DoraInfoGroup::CreateView(QWidget *parent)
{
  return new DoraInfoGroup(parent);
}

DoraClassObject *DoraInfoGroup::CreateDocument()
{
  return NULL;
}

void DoraInfoGroup::OnDraw(QPainter *p)
{
  p->setFont(font);
  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->setBrush(QBrush(QColor(192, 192, 192)));
  p->drawRect(clientRect);

  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->setBrush(QBrush(QColor(192, 192, 192)));
  m_pStaticCtrl1->show();
  m_pStaticCtrl2->show();
  m_pStaticCtrl3->show();
  m_pPushButton->show();

  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int wFont = sizeFont.width();
  int hFont = fm.height() - fm.descent();
  for(int i = 0; i <= 2; i++) {
    int len = strlen(infoStr[i].p);
    int x = itemRect[i].rect.x();
    int y = itemRect[i].rect.y() - 3;
    for(int j = 0; j <= len - 1; j++) {
      char *s = new char[2];
      s[0] = '\0';
      sprintf(s, "%c\0", infoStr[i].p[j]);
      QString qStr;
      qStr = "";qStr = convChar.getCharCodec(s);
      p->drawText(x, y, qStr);
      p->drawText(x+1, y, qStr);
      x += wFont;
      delete s;
    }
  }
}

void DoraInfoGroup::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);
  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int wFont = sizeFont.width();
  int hFont = fm.height() - fm.descent();
  int w = (wFont * 12) + 5;
  int h = hFont + 8;
  int x = 10;
  int y = hFont + 3;
  int maxElement = itemRect.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    int m = (i + 1) % 2;
    x = 10;
    if(m == 0)
      x = (w + 40);
    if((m != 0) && ((i + 1) >= 3))
      y = ((2 * hFont) + 3) + h;
    strRect item;
    item.rect.setX(x);
    item.rect.setY(y);
    item.rect.setWidth(w);
    item.rect.setHeight(h);
    itemRect.setElement(item, i + 1);
  }
  m_pStaticCtrl1->setGeometry(itemRect[0].rect);
  m_pStaticCtrl1->hide();
  m_pStaticCtrl2->setGeometry(itemRect[1].rect);
  m_pStaticCtrl2->hide();
  m_pStaticCtrl3->setGeometry(itemRect[2].rect);
  m_pStaticCtrl3->hide();
  m_pPushButton->setGeometry(itemRect[3].rect);
  m_pPushButton->hide();
}

void DoraInfoGroup::OnCreate(DoraClassObject *pObject, DoraBaseView *pView)
{
  m_pObject = pObject;
  m_pView   = pView;
  setFont(font);
  
  DoraCentralDocument *pDoc = GetDocument();
  
  strRect item;
  item.rect.setX(0);
  item.rect.setY(0);
  item.rect.setWidth(0);
  item.rect.setHeight(0);
  for(int i = 0; i <= 3; i++) {
    itemRect.setElement(item, i + 1);
  }

  DoraRuntimeClass *runtimePushButton = RUNTIME_CLASS(DoraPushButton);
  DoraClassObject *objectPushButton = runtimePushButton->CreateView(this);
  m_pPushButton = dynamic_cast<DoraPushButton *>(objectPushButton);
  m_pPushButton->convChar = pDoc->convChar;
  m_pPushButton->selectNewFont(font);
  m_pPushButton->OnCreate(m_pObject, this);
  m_pPushButton->ctrlID = DM_PUSHBUTTON;
  m_pPushButton->nID = ID_PUSHBUTTON_OK;
  Str35 cap1, cap2;
  cap1 = "";cap2 = "";
  cap1 = pDoc->capInfoButton1;;
  m_pPushButton->setCaption(cap1, cap2);
  m_pPushButton->hide();

  infoStr[0] = "";infoStr[0] = pDoc->capInfo1; // " :";
  infoStr[1] = "";infoStr[1] = pDoc->capInfo2; // " :";
  infoStr[2] = "";infoStr[2] = pDoc->capInfo3; // " :";

  DoraRuntimeClass *runtimeStaticCtrl1 = RUNTIME_CLASS(DoraStaticCtrl);
  DoraClassObject *objectStaticCtrl1 = runtimeStaticCtrl1->CreateView(this);
  m_pStaticCtrl1 = dynamic_cast<DoraStaticCtrl *>(objectStaticCtrl1);
  m_pStaticCtrl1->convChar = pDoc->convChar;
  m_pStaticCtrl1->selectNewFont(font);
  m_pStaticCtrl1->OnCreate(m_pObject, this);
  m_pStaticCtrl1->setColor(QColor(0, 0, 0), QColor(0, 0, 0), QColor(255, 255, 255));
  m_pStaticCtrl1->hide();

  DoraRuntimeClass *runtimeStaticCtrl2 = RUNTIME_CLASS(DoraStaticCtrl);
  DoraClassObject *objectStaticCtrl2 = runtimeStaticCtrl2->CreateView(this);
  m_pStaticCtrl2 = dynamic_cast<DoraStaticCtrl *>(objectStaticCtrl2);
  m_pStaticCtrl2->convChar = pDoc->convChar;
  m_pStaticCtrl2->selectNewFont(font);
  m_pStaticCtrl2->OnCreate(m_pObject, this);
  m_pStaticCtrl2->setColor(QColor(0, 0, 0), QColor(0, 0, 0), QColor(255, 255, 255));
  m_pStaticCtrl2->hide();

  DoraRuntimeClass *runtimeStaticCtrl3 = RUNTIME_CLASS(DoraStaticCtrl);
  DoraClassObject *objectStaticCtrl3 = runtimeStaticCtrl3->CreateView(this);
  m_pStaticCtrl3 = dynamic_cast<DoraStaticCtrl *>(objectStaticCtrl3);
  m_pStaticCtrl3->convChar = pDoc->convChar;
  m_pStaticCtrl3->selectNewFont(font);
  m_pStaticCtrl3->OnCreate(m_pObject, this);
  m_pStaticCtrl3->setColor(QColor(0, 0, 0), QColor(0, 0, 0), QColor(255, 255, 255));
  m_pStaticCtrl3->hide();

}

DoraCentralDocument *DoraInfoGroup::GetDocument()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralView *m_pCentralView = m_pMainFrame->GetCentralView();
  DoraCentralDocument *pDoc = m_pCentralView->GetDocument();
  return pDoc;
}

void DoraInfoGroup::PostMouseMessage(int ctrl, int index)
{
  DoraCentralDocument *pDoc = GetDocument();

  if(ctrl == DM_PUSHBUTTON) {
    if(index == ID_PUSHBUTTON_OK) {
      pDoc->calcCalendarDay(this);
    }
  }
}

void DoraInfoGroup::updateView()
{
  DoraCentralDocument *pDoc = GetDocument();
  pDoc->m_pCriticalSection->lock();
  Str35 str1, str2, str3;
  str1 = "";sprintf(str1.p, "%10d\0", pDoc->calDay);
  str2 = "";sprintf(str2.p, "%10d\0", pDoc->blcDay);
  str3 = "";sprintf(str3.p, "%10d\0", pDoc->redDay);
  m_pStaticCtrl1->setCaption(str1);
  m_pStaticCtrl1->repaint(false);
  m_pStaticCtrl2->setCaption(str2);
  m_pStaticCtrl2->repaint(false);
  m_pStaticCtrl3->setCaption(str3);
  m_pStaticCtrl3->repaint(false);
  pDoc->m_pCriticalSection->unlock();
}

void DoraInfoGroup::UpdateConvChar()
{
  DoraCentralDocument *pDoc = GetDocument();
  Str35 cap;
  cap = "";
  m_pPushButton->convChar = convChar;
  m_pPushButton->setCaption(pDoc->capInfoButton1, cap);
  m_pPushButton->repaint(false);
  m_pStaticCtrl1->convChar = convChar;
  m_pStaticCtrl1->repaint(false);
  m_pStaticCtrl2->convChar = convChar;
  m_pStaticCtrl2->repaint(false);
  m_pStaticCtrl3->convChar = convChar;
  m_pStaticCtrl3->repaint(false);
  infoStr[0] = "";infoStr[0] = pDoc->capInfo1;
  infoStr[1] = "";infoStr[1] = pDoc->capInfo2;
  infoStr[2] = "";infoStr[2] = pDoc->capInfo3;
  repaint(false);
}
