#include "DoraCentralDocument.h"
#include "DoraCentralView.h"
#include "DoraMainFrame.h"
#include "DoraLeftView.h"

DoraRuntimeClass DoraLeftView::classDoraLeftView={"DoraLeftView", DoraLeftView::CreateView, NULL};

DoraClassObject *DoraLeftView::CreateView(QWidget *parent)
{
  return new DoraLeftView(parent);
}

DoraClassObject *DoraLeftView::CreateDocument()
{
  return NULL;
}

void DoraLeftView::OnDraw(QPainter *p)
{
  p->setFont(font);
  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->setBrush(QBrush(QColor(192, 192, 192)));
  p->drawRect(clientRect);

  m_pClockView->show();
  m_pSpinGroup->show();
  m_pIntrGroup->show();
  m_pInfoGroup->show();
}

void DoraLeftView::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);

  int wScr = (int)((cx - 4) / 4);
  int wClock = wScr * 3;
  int hClock = wClock;
  int xClock = 2;
  int yClock = 2;
  clockRect.setX(xClock);
  clockRect.setY(yClock);
  clockRect.setWidth(wClock);
  clockRect.setHeight(hClock);
  
  int wGroup = wScr;
  int hGroup = wClock;
  int xGroup = 2 + wClock;
  int yGroup = 2;
  groupRect.setX(xGroup);
  groupRect.setY(yGroup);
  groupRect.setWidth(wGroup);
  groupRect.setHeight(hGroup);

  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int w = sizeFont.width();
  int h = fm.height() - fm.descent();
  
  int xIntr = xClock;
  int yIntr = 4 + hClock;
  int wIntr = wScr * 4;
  int hIntr = (h * 4) + 8;
  grIntrRect.setX(xIntr);
  grIntrRect.setY(yIntr);
  grIntrRect.setWidth(wIntr);
  grIntrRect.setHeight(hIntr);

  int xInfo = xClock;
  int yInfo = yIntr + hIntr + 2;
  int wInfo = wScr * 4;
  int hInfo = (h * 4) + 40;
  grInfoRect.setX(xInfo);
  grInfoRect.setY(yInfo);
  grInfoRect.setWidth(wInfo);
  grInfoRect.setHeight(hInfo);

  m_pClockView->setGeometry(clockRect);
  m_pClockView->hide();
  m_pSpinGroup->setGeometry(groupRect);
  m_pSpinGroup->hide();
  m_pIntrGroup->setGeometry(grIntrRect);
  m_pIntrGroup->hide();
  m_pInfoGroup->setGeometry(grInfoRect);
  m_pInfoGroup->hide();
}

void DoraLeftView::OnCreate(DoraClassObject *pObject, DoraBaseView *pView)
{
  m_pObject = pObject;
  m_pView   = pView;
  setFont(font);

  DoraCentralDocument *pDoc = GetDocument();

  DoraRuntimeClass *runtimeClock = RUNTIME_CLASS(DoraClockView);
  DoraClassObject *objectClock = runtimeClock->CreateView(this);
  m_pClockView = dynamic_cast<DoraClockView *>(objectClock);
  m_pClockView->convChar = pDoc->convChar;
  m_pClockView->selectNewFont(pDoc->getDefaultFont(2));
  m_pClockView->OnCreate(m_pObject, this);
  m_pClockView->hide();

  DoraRuntimeClass *runtimeSpinGroup = RUNTIME_CLASS(DoraSpinGroup);
  DoraClassObject *objectSpinGroup = runtimeSpinGroup->CreateView(this);
  m_pSpinGroup = dynamic_cast<DoraSpinGroup *>(objectSpinGroup);
  m_pSpinGroup->convChar = pDoc->convChar;
  m_pSpinGroup->selectNewFont(pDoc->getDefaultFont(5));
  m_pSpinGroup->OnCreate(m_pObject, this);
  m_pSpinGroup->hide();

  DoraRuntimeClass *runtimeIntrGroup = RUNTIME_CLASS(DoraIntrGroup);
  DoraClassObject *objectIntrGroup = runtimeIntrGroup->CreateView(this);
  m_pIntrGroup = dynamic_cast<DoraIntrGroup *>(objectIntrGroup);
  m_pIntrGroup->convChar = pDoc->convChar;
  m_pIntrGroup->selectNewFont(pDoc->getDefaultFont(4));
  m_pIntrGroup->OnCreate(m_pObject, this);
  m_pIntrGroup->hide();

  DoraRuntimeClass *runtimeInfoGroup = RUNTIME_CLASS(DoraInfoGroup);
  DoraClassObject *objectInfoGroup = runtimeInfoGroup->CreateView(this);
  m_pInfoGroup = dynamic_cast<DoraInfoGroup *>(objectInfoGroup);
  m_pInfoGroup->convChar = pDoc->convChar;
  m_pInfoGroup->selectNewFont(pDoc->getDefaultFont(4));
  m_pInfoGroup->OnCreate(m_pObject, this);
  m_pInfoGroup->hide();

  m_pClockView->startTimer(1000);
}

DoraCentralDocument *DoraLeftView::GetDocument()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralView *m_pCentralView = m_pMainFrame->GetCentralView();
  DoraCentralDocument *pDoc = m_pCentralView->GetDocument();

  return pDoc;
}

void DoraLeftView::UpdateTimeView()
{
  m_pSpinGroup->UpdateTimeView();
}

void DoraLeftView::stopClock()
{
  m_pClockView->stopTimer();
}

void DoraLeftView::startClock()
{
  m_pClockView->startTimer(1000);
}

void DoraLeftView::UpdateConvChar()
{
  m_pSpinGroup->convChar = convChar;
  m_pSpinGroup->UpdateConvChar();
  m_pIntrGroup->convChar = convChar;
  m_pIntrGroup->UpdateConvChar();
  m_pInfoGroup->convChar = convChar;
  m_pInfoGroup->UpdateConvChar();
}
