#include "DoraMainApp.h"
#include "DoraMainFrame.h"

DoraRuntimeClass DoraMainFrame::classDoraMainFrame = {"DoraMainFrame", DoraMainFrame::CreateView, NULL};

DoraClassObject *DoraMainFrame::CreateView(QWidget *parent) 
{
  return new DoraMainFrame(parent);
}

DoraClassObject *DoraMainFrame::CreateDocument()
{
  return NULL;
}

void DoraMainFrame::OnCreate(DoraClassObject *pObject)
{
  m_pObject = pObject;
  
  DoraRuntimeClass *runtimeInstallClass = RUNTIME_CLASS(DoraInstallClass);
  DoraClassObject *objectInstallClass = runtimeInstallClass->CreateDocument();
  m_pInstallClass = dynamic_cast<DoraInstallClass *>(objectInstallClass);
  m_pInstallClass->initDocument();
  
  DoraRuntimeClass    *runtimeDocument = RUNTIME_CLASS(DoraCentralDocument);
  DoraClassObject     *objectDocument  = runtimeDocument->CreateDocument();
  DoraCentralDocument *centralDocument = dynamic_cast<DoraCentralDocument *>(objectDocument);
  centralDocument->initDocument();
  baseDocument = dynamic_cast<DoraBaseDocument *>(centralDocument);
  selectNewFont(centralDocument->getDefaultFont(5));
  setFont(centralDocument->getDefaultFont(5));
  convChar = centralDocument->convChar;

  DoraRuntimeClass *runtimeView = RUNTIME_CLASS(DoraCentralView);
  DoraClassObject  *objectView  = runtimeView->CreateView(this);
  m_pCentralView = dynamic_cast<DoraCentralView *>(objectView);
  m_pCentralView->selectNewFont(centralDocument->getDefaultFont(5));
  m_pCentralView->convChar = centralDocument->convChar;
  m_pCentralView->OnCreate(this);
  m_pCentralView->hide();

  DoraRuntimeClass *runtimeMenu = RUNTIME_CLASS(DoraGlobMenuView);
  DoraClassObject *objectMenu = runtimeMenu->CreateView(this);
  m_pGlobMenuView = dynamic_cast<DoraGlobMenuView *>(objectMenu);
  m_pGlobMenuView->selectNewFont(centralDocument->getDefaultFont(4));
  m_pGlobMenuView->convChar = centralDocument->convChar;
  m_pGlobMenuView->OnCreate(this);
  m_pGlobMenuView->hide();
  m_pGlobMenuView->setGlobalMenu();

  DoraRuntimeClass *runtimeRekl = RUNTIME_CLASS(DoraReklView);
  DoraClassObject *objectRekl = runtimeRekl->CreateView(this);
  m_pReklView = dynamic_cast<DoraReklView *>(objectRekl);
  m_pReklView->selectNewFont(centralDocument->getDefaultFont(4));
  m_pReklView->convChar = centralDocument->convChar;
  m_pReklView->OnCreate(this);
  m_pReklView->hide();

  m_pReklView->startTimer(500);
}

void DoraMainFrame::OnDestroy()
{
  m_pCentralView->DestroyView();
}

void DoraMainFrame::OnDraw(QPainter *p)
{
  m_pGlobMenuView->show();
  m_pCentralView->show();
  m_pReklView->show();
}

void DoraMainFrame::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);

  QRect menuRect;
  menuRect.setX(0);
  menuRect.setY(0);
  menuRect.setWidth(cx);
  menuRect.setHeight(24);
  m_pGlobMenuView->setGeometry(menuRect);
  m_pGlobMenuView->hide();

  QRect centralRect;
  centralRect.setX(0);
  centralRect.setY(25);
  centralRect.setWidth(cx);
  centralRect.setHeight(cy - 50);
  m_pCentralView->setGeometry(centralRect);
  m_pCentralView->hide();

  QRect reklRect;
  reklRect.setX(0);
  reklRect.setY((menuRect.height() + centralRect.height()) + 2);
  reklRect.setWidth(cx);
  reklRect.setHeight(24);
  m_pReklView->setGeometry(reklRect);
  m_pReklView->hide();

  xGMenu = menuRect.x();
  yGMenu = menuRect.y();
  wGMenu = menuRect.width();
  hGMenu = menuRect.height();

  xCView = centralRect.x();
  yCView = centralRect.y();
  wCView = centralRect.width();
  hCView = centralRect.height();

  xSView = reklRect.x();
  ySView = reklRect.y();
  wSView = reklRect.width();
  hSView = reklRect.height();
}

DoraCentralDocument *DoraMainFrame::GetDocument()
{
  DoraCentralDocument *pDoc = dynamic_cast<DoraCentralDocument *>(baseDocument);
  return pDoc;
}

DoraCentralView *DoraMainFrame::GetCentralView()
{
  if(m_pCentralView != NULL)
    return m_pCentralView;
  else
    return NULL;
}

void DoraMainFrame::OnCloseView()
{
  printf(" OnCloseView   DoraMainFrame\n");
  DoraMainApp *m_pMainApp = dynamic_cast<DoraMainApp *>(m_pObject);
  m_pMainApp->quit();
}

void DoraMainFrame::OnAbautView()
{
  DoraCentralDocument *pDoc = GetDocument();

  if(m_pAbautView == NULL) {
    DoraRuntimeClass *runtimeAbautView = RUNTIME_CLASS(DoraAbautView);
    DoraClassObject *objectAbautView = runtimeAbautView->CreateView(this);
    m_pAbautView = dynamic_cast<DoraAbautView *>(objectAbautView);
    m_pAbautView->selectNewFont(pDoc->getDefaultFont(4));
    m_pAbautView->convChar = pDoc->convChar;
    m_pAbautView->OnCreate(this, this);
  }

  int w = (int)(3 * (clientRect.width() / 4));
  int h = (int)(3 * (clientRect.height() / 4));

  int x = (int)((clientRect.x() + (clientRect.width() - w)) / 2);
  int y = (int)((clientRect.y() + (clientRect.height() - h)) / 2);

  QRect rect;
  rect.setX(x);
  rect.setY(y);
  rect.setWidth(w);
  rect.setHeight(h);

  m_pAbautView->setGeometry(rect);
  m_pAbautView->show();
}

void DoraMainFrame::closeAbautView()
{
  if(m_pAbautView != NULL) {
    m_pAbautView->hide();
    delete m_pAbautView;
    m_pAbautView = NULL;
  }
}

void DoraMainFrame::PostMouseMessage(int ctrl, int index)
{
  if(ctrl == ID_SIGNAL_CLOSE_ABAUTVIEW)
    closeAbautView();
  if(ctrl == ID_PUSHBUTTON_OK)
    closeEditDialog();
}

void DoraMainFrame::OnReklDialog()
{
  DoraCentralDocument *pDoc = GetDocument();

  if(m_pEditDialog == NULL) {
    DoraRuntimeClass *runtimeDialog = RUNTIME_CLASS(DoraEditDialog);
    DoraClassObject *objectDialog = runtimeDialog->CreateView(this);
    m_pEditDialog = dynamic_cast<DoraEditDialog *>(objectDialog);
    m_pEditDialog->selectNewFont(pDoc->getDefaultFont(4));
    m_pEditDialog->convChar = convChar;
    m_pEditDialog->OnCreate(this, this);
    m_pEditDialog->hide();
    m_pEditDialog->enablePushButton(ID_PUSHBUTTON_OK, true);
    m_pEditDialog->enablePushButton(ID_PUSHBUTTON_SAVE, false);
    m_pEditDialog->caption = "";
    m_pEditDialog->caption = pDoc->capReclView;
  }
  
  int w = (int)(3 * (clientRect.width() / 3)) - 20;
  int h = (int)(3 * (clientRect.height() / 3)) - 70;

  int x = (int)((clientRect.x() + (clientRect.width() - w)) / 2);
  int y = (int)((clientRect.y() + (clientRect.height() - h)) / 2);

  QRect rect;
  rect.setX(x);
  rect.setY(y);
  rect.setWidth(w);
  rect.setHeight(h);

  m_pEditDialog->setGeometry(rect);
  m_pEditDialog->show();
}

void DoraMainFrame::closeEditDialog()
{
  if(m_pEditDialog != NULL) {
    m_pEditDialog->hide();
    m_pEditDialog->DestroyDialog();
    delete m_pEditDialog;
    m_pEditDialog = NULL;
  }
  m_pReklView->stopTimer();
  m_pReklView->reloadReclams();
  m_pReklView->startTimer(500);
}

void DoraMainFrame::UpdateConvChar()
{
  DoraCentralDocument *pDoc = GetDocument();
  Str35 prefix;
  prefix = "";prefix = pDoc->itemLangName[pDoc->curentLang - 1].prefix;
  pDoc->loadOtherCaption(prefix);
  convChar = pDoc->convChar;
  m_pCentralView->convChar = convChar;
  m_pCentralView->UpdateConvChar();;
  if(m_pAbautView != NULL) {
    m_pAbautView->hide();
    delete m_pAbautView;
    m_pAbautView = NULL;
  }
  if(m_pEditDialog != NULL) {
    m_pEditDialog->hide();
    m_pEditDialog->DestroyDialog();
    delete m_pEditDialog;
    m_pEditDialog = NULL;
  }
  if(m_pCentralView != NULL) {
    m_pCentralView->closeBioritm();
  }
}

void DoraMainFrame::OnBioritms()
{
  m_pCentralView->setBioritms();
}
