#include "DoraCentralDocument.h"
#include "DoraMainFrame.h"
#include "DoraCentralView.h"
#include "DoraPopupMenu.h"

DoraRuntimeClass DoraPopupMenu::classDoraPopupMenu={"DoraPopupMenu", DoraPopupMenu::CreateView, NULL};

DoraClassObject *DoraPopupMenu::CreateView(QWidget *parent)
{
  return new DoraPopupMenu(parent);
}

DoraClassObject *DoraPopupMenu::CreateDocument()
{
  return NULL;
}

void DoraPopupMenu::OnCreate(DoraClassObject *pObject, DoraBaseView *pView)
{
  m_pObject = pObject;
  m_pView   = pView;
}

void DoraPopupMenu::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);
  int maxElement = itemPopup.maxElement();
  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int wf = sizeFont.width();
  int hf = fm.height() - fm.descent();
  int x = clientRect.x() + 2;
  int y = clientRect.y() + 2;
  int w = clientRect.width() - 5;
  int h = hf + 4;
  for(int i = 0; i <= maxElement - 1; i++) {
    QRect rect;
    rect.setX(x);
    rect.setY(y);
    rect.setWidth(w);
    if(itemPopup[i].nID == ID_STATUS_SEPARATOR)
      h = 9;
    else
      h = hf + 4;
    rect.setHeight(h);
    itemPopup[i].staticCtrl->setGeometry(rect);
    itemPopup[i].staticCtrl->hide();
    y += h;
  }
}

void DoraPopupMenu::OnDraw(QPainter *p)
{
  p->setFont(font);
  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->setBrush(QBrush(QColor(192, 192, 192)));
  p->drawRect(clientRect);

  int x1 = clientRect.x() + 1;
  int y1 = clientRect.y() + 1;
  int x2 = clientRect.width() - 2;
  int y2 = clientRect.height() - 2;
  p->setPen(QPen(QColor(255, 255, 255), 1));
  p->drawLine(x1, y2, x1, y1);
  p->drawLine(x1, y1, x2, y1);
  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->drawLine(x1, y2, x2, y2);
  p->drawLine(x2, y2, x2, y1);

  int maxElement = itemPopup.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
      itemPopup[i].staticCtrl->show();
  }
}

void DoraPopupMenu::setPopupItem(const int index)
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = m_pMainFrame->GetDocument();
  Str35 prefix;
  prefix = "";prefix = pDoc->itemLangName[pDoc->curentLang - 1].prefix;
  attr.setPopupMenu(index, prefix);
  itemPopup.removeAllElement();
  strItem item;
  int maxElement = attr.getMaxEPopupItem();
  for(int i = 0; i <= maxElement - 1; i++) {
    DoraRuntimeClass *runtimeStaticCtrl = RUNTIME_CLASS(DoraStaticCtrl);
    DoraClassObject *objectStaticCtrl = runtimeStaticCtrl->CreateView(this);
    item.staticCtrl = dynamic_cast<DoraStaticCtrl *>(objectStaticCtrl);
    item.staticCtrl->selectNewFont(font);
    item.staticCtrl->convChar = convChar;
    item.staticCtrl->OnCreate(m_pObject, this);
    item.staticCtrl->hide();
    item.staticCtrl->setColor(QColor(0, 0, 0), QColor(192, 192, 192), QColor(192, 192, 192));
    item.staticCtrl->setSelectColor(QColor(255, 255, 255), QColor(0, 0, 255), QColor(0, 0, 255));
    item.staticCtrl->setCaption(attr.itemPopupMenu[i].strMenu);
    item.staticCtrl->nID = attr.itemPopupMenu[i].nID;
    item.staticCtrl->status = attr.itemPopupMenu[i].itemType;
    item.nID = attr.itemPopupMenu[i].itemType;
    itemPopup.setElement(item, i + 1);
  }
}

void DoraPopupMenu::PostMouseMove(int index)
{
  printf(" PostMouseMove   DoraPopupMenu -> %4d\n", index);
  int maxElement = itemPopup.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    if(itemPopup[i].staticCtrl->nID != index) {
      itemPopup[i].staticCtrl->enableSelected(false);
      itemPopup[i].staticCtrl->repaint(false);
      oldIndex = i + 1;
    }
  }
}

void DoraPopupMenu::PostMouseMessage(int ctrl, int index)
{
   printf(" PostMouseMessage   DoraPopupMenu -> %4d\n", index);
   int ctrlID = ID_SIGNAL_POPUPMENU;
   m_pView->PostMouseMessage(ctrlID, index);
}
