#include "DoraCentralDocument.h"
#include "DoraCentralView.h"
#include "DoraMainFrame.h"
#include "DoraDialogDoc.h"
#include "DoraReklView.h"

DoraRuntimeClass DoraReklView::classDoraReklView={"DoraReklView", DoraReklView::CreateView, NULL};

DoraClassObject *DoraReklView::CreateView(QWidget *parent)
{
  return new DoraReklView(parent);
}

DoraClassObject *DoraReklView::CreateDocument()
{
  return NULL;
}

void DoraReklView::OnDraw(QPainter *p)
{
  p->setFont(font);
  setPixmap();
  int x1 = clientRect.x();
  int y1 = clientRect.y();
  int x2 = x1 + clientRect.width();
  int y2 = y1 + clientRect.height();
  bitBlt(this, x1, y1, &pixmap, x1, y1, x2 - x1, y2 - y1);
}

void DoraReklView::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);
}

void DoraReklView::OnCreate(DoraClassObject *pObject)
{
  m_pObject = pObject;
  setFont(font);
  DoraDialogDoc *pDoc = GetDialogDoc();
  int index = pDoc->curentReclams;
  pDoc->reklStr = "";pDoc->reklStr = pDoc->getReclString(index);
  convChar = DoraConvChar(pDoc->encodingName.p);
}

DoraCentralDocument *DoraReklView::GetDocument()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = m_pMainFrame->GetDocument();
  return pDoc;
}

DoraDialogDoc *DoraReklView::GetDialogDoc()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralView *m_pCentralView = m_pMainFrame->GetCentralView();
  DoraDialogDoc *pDoc = m_pCentralView->GetDialogDoc();
  return pDoc;
}

void DoraReklView::setPixmap()
{
  QPainter p;
  pixmap.resize(clientRect.width(), clientRect.height());
  pixmap.fill(QColor(255, 255, 255));
  p.begin(&pixmap);
  {
    p.setFont(font);
    p.setPen(QPen(QColor(0, 0, 0), 1));
    p.setBrush(QBrush(QColor(192, 192, 192)));
    p.drawRect(clientRect);
    int x1 = clientRect.x() + 2;
    int y1 = clientRect.y() + 2;
    int x2 = clientRect.width() - 4;
    int y2 = clientRect.height() - 4;
    
    p.setPen(QPen(QColor(0, 0, 0), 1));
    p.drawLine(x1, y2, x1, y1);
    p.drawLine(x1, y1, x2, y1);
    p.setPen(QPen(QColor(255, 255, 255)));
    p.drawLine(x1, y2, x2, y2);
    p.drawLine(x2, y2, x2, y1);
    
    p.setPen(QPen(QColor(0, 0, 0), 1));
    p.setBrush(QBrush(QColor(0, 0, 255)));
    x1 = clientRect.x() + 3;
    y1 = clientRect.y() + 3;
    x2 = clientRect.width() - 6;
    y2 = clientRect.height() - 6;
    QRect rect;
    rect.setX(x1);
    rect.setY(y1);
    rect.setWidth(x2);
    rect.setHeight(y2);
    p.drawRect(rect);

    QFontMetrics fm(font);
    QSize fontSize = fm.size(SingleLine, "H", 1);
    int w = fontSize.width();
    int h = fm.height() - fm.descent();
    DoraCentralDocument *pDoc = GetDocument();
    pDoc->setReklamaStr();
    maxCol = ((int)(rect.width() / w)) - 1;
    int x = rect.x() + 2;
    int y = (int)((clientRect.height() - h) / 2) + (h - 2);
    p.setPen(QPen(QColor(255, 255, 255), 1));
    CString str;
    str = "";str = getVisualStr();
    int len = strlen(str.p);
    for(int i = 0; i <= len - 1; i++) {
      char *s = new char[2];
      s[0] = '\0';
      sprintf(s, "%c\0", str.p[i]);
      QString qStr;
      qStr = "";qStr = convChar.getCharCodec(s);
      p.drawText(x, y, qStr);
      p.drawText(x + 1, y, qStr);
      x += w;
      delete s;
    }
  }
  p.end();
}

void DoraReklView::timerEvent(QTimerEvent *e)
{
  timID = e->timerId();
  killedTimer = false;
  OnClock();
}

void DoraReklView::OnClock()
{
  repaint(false);
}

CString DoraReklView::getVisualStr()
{
  DoraDialogDoc *pDoc = GetDialogDoc();
  CString reclStr;
  reclStr = "";reclStr = pDoc->reklStr.p;
  int len = strlen(reclStr.p);
  CString str;
  str = "";
  for(int i = col; i <= (col + maxCol) - 1; i++) {
    str = str + copyStr(reclStr, i, 1);
  }
  if((col + maxCol) >= len) {
    col = 0;
    int index = pDoc->curentReclams;
    pDoc->reklStr = "";pDoc->reklStr = pDoc->getReclString(index);
    convChar = DoraConvChar(pDoc->encodingName.p);
  }
  col++;

  return str;
}

void DoraReklView::reloadReclams()
{
  DoraDialogDoc *pDoc = GetDialogDoc();
  pDoc->setDefaultReclStr();
  col = 0;
  int index = pDoc->curentReclams;
  pDoc->reklStr = "";pDoc->reklStr = pDoc->getReclString(index);
  convChar = DoraConvChar(pDoc->encodingName.p);
}

void DoraReklView::stopTimer()
{
  if(!killedTimer) {
    killTimer(timID);
    killedTimer = true;
  }
}
