#include "DoraScrBarButton.h"
#include "resurce.h"

DoraRuntimeClass DoraScrBarButton::classDoraScrBarButton = {"DoraScrBarButton", DoraScrBarButton::CreateView, NULL};

DoraClassObject *DoraScrBarButton::CreateView(QWidget *parent)
{
  return new DoraScrBarButton(parent);
}

DoraClassObject *DoraScrBarButton::CreateDocument()
{
  return NULL;
}

void DoraScrBarButton::OnCreate(DoraClassObject *pObject, DoraBaseView *pView, const int style = ID_SCROLLBARBUTTON_UP)
{
  m_pObject = pObject;
  m_pView   = pView;
  printf(" OnCreate  DoraScrBarButton -> ViewClass\n");
  styleButton = style;
  OkPushed = FALSE;
  switch(styleButton) {
  case ID_SCROLLBARBUTTON_UP:
    postButton = POST_SCROLLBARBUTTON_UP;
    break;
  case ID_SCROLLBARBUTTON_DOWN :
    postButton = POST_SCROLLBARBUTTON_DOWN;
    break;
  case ID_SCROLLBARBUTTON_LEFT :
    postButton = POST_SCROLLBARBUTTON_LEFT;
    break;
  case ID_SCROLLBARBUTTON_RIGHT :
    postButton = POST_SCROLLBARBUTTON_RIGHT;
    break;
  }
}

void DoraScrBarButton::OnDraw(QPainter *p)
{
  setPixmap();
  int x1 = clientRect.x();
  int y1 = clientRect.y();
  int x2 = x1 + clientRect.width();
  int y2 = y1 + clientRect.height();

  bitBlt(this, x1, y1, &pm, x1, y1, x2 - x1, y2 - y1);
}

void DoraScrBarButton::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);
  printf("w = %4d, h = %4d\n", clientRect.width(), clientRect.height());

}

void DoraScrBarButton::setPixmap()
{
  QPainter p;
  int w = clientRect.width();
  int h = clientRect.height();
  pm.resize(w, h);
  pm.fill(QColor(255, 255, 255));
  p.begin(&pm);
  {
    p.setPen(QColor(0, 0, 0));
    p.setBrush(QColor(192, 192, 192));
    p.drawRect(clientRect);
    int x1 = clientRect.x() + 1;
    int y1 = clientRect.y() + 1;
    int x2 = (x1 + clientRect.width()) - 3;
    int y2 = (y1 + clientRect.height()) - 3;
    if(!OkPushed) {
      p.setPen(QColor(255, 255, 255));
      p.drawLine(x1, y2, x1, y1);
      p.drawLine(x1, y1, x2, y1);
      p.setPen(QColor(0, 0, 0));
      p.drawLine(x2, y1, x2, y2);
      p.drawLine(x2, y2, x1, y2);
    } else {
      p.setPen(QColor(0, 0, 0));
      p.drawLine(x1, y2, x1, y1);
      p.drawLine(x1, y1, x2, y1);
      p.setPen(QColor(255, 255, 255));
      p.drawLine(x2, y1, x2, y2);
      p.drawLine(x2, y2, x1, y2);
    }
    switch(styleButton) {
    case ID_SCROLLBARBUTTON_UP    :
      DrawUpButton(&p);
      break;
    case ID_SCROLLBARBUTTON_DOWN  :
      DrawDownButton(&p);
      break;
    case ID_SCROLLBARBUTTON_LEFT  :
      DrawLeftButton(&p);
      break;
    case ID_SCROLLBARBUTTON_RIGHT :
      DrawRightButton(&p);
      break;
    }

  }
  p.end();
}

void DoraScrBarButton::DrawLeftButton(QPainter *p)
{
  QPixmap pmap(bmp_scrbutton_left);
  QSize size = pmap.size();
  int xp1 = (int)((clientRect.width() - size.width())/2);
  int yp1 = (int)((clientRect.height() - size.height())/2);
  if(OkPushed)
    xp1++;
  p->drawPixmap(xp1, yp1, pmap);
}

void DoraScrBarButton::DrawRightButton(QPainter *p)
{
  QPixmap pmap(bmp_scrbutton_right);
  QSize size = pmap.size();
  int xp1 = (int)((clientRect.width() - size.width())/2);
  int yp1 = (int)((clientRect.height() - size.height())/2);
  if(OkPushed)
    xp1++;
  p->drawPixmap(xp1, yp1, pmap);
}

void DoraScrBarButton::DrawUpButton(QPainter *p)
{
  QPixmap pmap(bmp_scrbutton_up);
  QSize size = pmap.size();
  printf("w = %4d, h = %4d\n", size.width(), size.height());
  int xp1 = (int)((clientRect.width() - size.width())/2);
  int yp1 = (int)((clientRect.height() - size.height())/2);
  if(OkPushed)
    xp1++;
  p->drawPixmap(xp1, yp1, pmap);

}

void DoraScrBarButton::DrawDownButton(QPainter *p)
{
  QPixmap pmap(bmp_scrbutton_down);
  QSize size = pmap.size();
  int xp1 = (int)((clientRect.width() - size.width())/2);
  int yp1 = (int)((clientRect.height() - size.height())/2);
  if(OkPushed)
    xp1++;
  p->drawPixmap(xp1, yp1, pmap);
}

void DoraScrBarButton::MouseLeftButton(int xLocPos, int yLocPos, int xPosGlb, int yPosGlb)
{
  OkPushed = TRUE;
  repaint();
  if(m_pView != NULL)
    m_pView->PostMouseMessage(postButton, styleButton);
}

void DoraScrBarButton::MouseRelease()
{
  OkPushed = FALSE;
  repaint();
}















