#include "DoraCentralDocument.h"
#include "DoraCentralView.h"
#include "DoraMainFrame.h"
#include "DoraScrollBox.h"

DoraRuntimeClass DoraScrollBox::classDoraScrollBox={"DoraScrollBox", DoraScrollBox::CreateView, NULL};

DoraClassObject *DoraScrollBox::CreateView(QWidget *parent)
{
  return new DoraScrollBox(parent);
}

DoraClassObject *DoraScrollBox::CreateDocument()
{
  return NULL;
}

void DoraScrollBox::OnDraw(QPainter *p)
{
  p->setFont(font);
  setPixmap();
  int x1 = clientRect.x();
  int y1 = clientRect.y();
  int x2 = x1 + clientRect.width();
  int y2 = y1 + clientRect.height();
  bitBlt(this, x1, y1, &pmap, x1, y1, x2 - x1, y2 - y1);
}

void DoraScrollBox::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);

  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int w = sizeFont.width();
  int h = fm.height() - fm.descent();
  int maxElement = itemData.maxElement();
  int xItem = 2;
  int yItem = 1;
  int wItem = cx - 5;
  int hItem = (h + 4);
  int j = iMin;
  for(int i = 0; i <= maxRow - 1; i++) {
    ItemRectStruct item;
    item.name = "";
    item.index = i + 1;
    item.isSelected = false;
    if(j <= maxElement) {
      item.name = itemData[j-1].name;
      item.index = itemData[j-1].index;
      if(j == curentIndex)
	item.isSelected = true;
    }
    item.rect.setX(xItem);
    item.rect.setY(yItem);
    item.rect.setWidth(wItem);
    item.rect.setHeight(hItem);
    itemRect.setElement(item, i + 1);
    yItem += (h + 3);j++;
  }
}

void DoraScrollBox::OnCreate(DoraClassObject *pObject, DoraBaseView *pView)
{
  m_pObject = pObject;
  m_pView   = pView;
  setFont(font);
  oldIndex = 1;
  setMouseTracking(TRUE);
}

void DoraScrollBox::setDataElement(Str35 name, const int index = 1)
{
  ItemDataStruct item;
  item.name = "";item.name = name;
  item.index = index;
  itemData.setElement(item, index);
}

void DoraScrollBox::MouseMove()
{
  int maxElement = itemRect.maxElement();

  for(int i = 0; i <= maxRow - 1; i++) {
    ItemRectStruct item;
    item = itemRect[i];
    item.isSelected = FALSE;
    if((yMovePos >= itemRect[i].rect.y()) && (yMovePos <= (itemRect[i].rect.y() + itemRect[i].rect.height()))) {
      item.isSelected = TRUE;
      curentIndex = iPos = item.index;
      row = i + 1;
    }
    itemRect.setElement(item, i + 1);
  }
  repaint(FALSE);
}

void DoraScrollBox::PostMouseClick(int index)
{
  int maxElement = itemRect.maxElement();
  int sPos = 1;
  for(int i = 0; i <= maxRow - 1; i++) {
    ItemRectStruct item;
    item = itemRect[i];
    item.isSelected = FALSE;
    if((yMovePos >= itemRect[i].rect.y()) && (yMovePos <= (itemRect[i].rect.y() + itemRect[i].rect.height()))) {
      item.isSelected = TRUE;
      sPos = item.index;
    }
    itemRect.setElement(item, i + 1);
  }
  int maxData = itemData.maxElement();
  if(sPos <= 0)
    sPos = 1;
  if(sPos >= maxData)
    sPos = maxData;
  if(m_pView != NULL)
    m_pView->PostMouseMessage(POST_SCROLLBOX, sPos);
}

void DoraScrollBox::setCheckElement(const int index)
{
  int maxElement = itemRect.maxElement();
  if(maxElement >= 1) {
    for(int i = 0; i <= maxElement - 1; i++) {
      ItemRectStruct item;
      item = itemRect[i];
      item.isSelected = false;
      if(itemRect[i].index == index)
	item.isSelected = true;
      itemRect.setElement(item, index);
    }
    curentIndex = index;
    oldIndex = index;
  }
}

void DoraScrollBox::setPixmap()
{
  QPainter p;
  pmap.resize(clientRect.width(), clientRect.height());
  pmap.fill(QColor(255, 255, 255));
  p.begin(&pmap);
  {
     p.setPen(QPen(QColor(0, 0, 0), 1));
     p.setBrush(QBrush(QColor(255, 255, 255)));
     p.drawRect(clientRect);
     QFontMetrics fm(font);
     QSize sizeFont = fm.size(SingleLine, "H", 1);
     int w = sizeFont.width();
     int h = fm.height() - fm.descent();
     p.setFont(font);
     int xPos = 2;
     int yPos = h + 4;
     int maxElement = itemRect.maxElement();
     for(int i = 0; i <= maxRow - 1; i++) {
       xPos = 3;
       yPos = (itemRect[i].rect.y() + h);
       int len = strlen(itemRect[i].name.p);
       if(itemRect[i].isSelected) {
	 p.setPen(QPen(QColor(0, 0, 255), 1));
	 p.setBrush(QBrush(QColor(0, 0, 255)));
	 p.drawRect(itemRect[i].rect);
	 p.setPen(QPen(QColor(255, 255, 255), 1));
       } else {
	 p.setPen(QPen(QColor(255, 255, 255), 1));
	 p.setBrush(QBrush(QColor(255, 255, 255)));
	 p.drawRect(itemRect[i].rect);
	 p.setPen(QPen(QColor(0, 0, 0), 1));
       }
       for(int j = 0; j <= len - 1; j++) {
	 char *s = new char[2];
	 s[0] = '\0';
	 sprintf(s, "%c\0", itemRect[i].name.p[j]);
	 QString qStr;
	 qStr = "";qStr = convChar.getCharCodec(s);
	 p.drawText(xPos, yPos, qStr);
	 xPos += w;
	 delete s;
       }
     }
  }
  p.end();
}

void DoraScrollBox::scrollUp()
{
  int maxElement = itemData.maxElement();
  if((iPos <= 1) && (row <= 1)) {
    iPos = 1;row = 1;
  }
  if((iPos > 1) && (row <= 1)) {
    iPos--;row = 1;
    iMin = iPos;iMax = (iMin + maxRow) - 1;
    if(iMax >= maxElement)
      iMax = maxElement;
  }
  if((iPos > 1) && (row > 1)) {
    iPos--;row--;
  }
  curentIndex = iPos;
  reloadData();
}

void DoraScrollBox::scrollDown()
{
  int maxElement = itemData.maxElement();
  if((iPos >= maxElement) && (row >= maxRow)) {
    iPos = maxElement;row = maxRow;
  }
  if((iPos < maxElement) && (row >= maxRow)) {
    iPos++;row = maxRow;
    iMax = iPos;iMin = (iMax - maxRow) + 1;
    if(iMin <= 1)
      iMin = 1;
  }
  if((iPos < maxElement) && (row < maxRow)) {
    iPos++;row++;
  }
  curentIndex = iPos;
  reloadData();
}

void DoraScrollBox::setRectElement(const int index)
{
  int maxElement = itemData.maxElement();
  iPos = 1;
  for(int i = 0; i <= maxElement - 1; i++) {
    if(itemData[i].index == index) {
      iPos = itemData[i].index;
    }
  }
  if(maxRow >= maxElement) {
    iMin = 1;
    iMax = maxElement;
  } else {
    iMin = index - 5;
    if(iMin <= 1)
      iMin = 1;
    iMax = iMin + maxRow;
    if(iMax >= maxElement)
      iMax = maxElement;
  }
  int j = iMin;
  itemRect.removeAllElement();
  for(int i = 0; i <= maxRow - 1; i++) {
    ItemRectStruct item;
    item.name = "";
    item.isSelected = false;
    if(j <= maxElement) {
      item.name = itemData[j-1].name;
      item.index = itemData[j-1].index;
      if(j == index) {
	item.isSelected = true;
	iPos = j;row = i + 1;
      }
      itemRect.setElement(item, i + 1);
    }
    j++;
  }
  curentIndex = index;
  repaint(false);
}

void DoraScrollBox::reloadData()
{
  int maxElement = itemData.maxElement();
  int j = iMin;
  for(int i = 0; i <= maxRow - 1; i++) {
    ItemRectStruct item;
    item = itemRect[i];
    item.name = "";
    item.isSelected = false;
    if(j <= maxElement) {
      item.name = itemData[j-1].name;
      item.index = itemData[j-1].index;
      if(j == curentIndex)
	item.isSelected = true;
    }
    itemRect.setElement(item, i + 1);j++;
  }
  repaint(false);
}

void DoraScrollBox::UpdateConvChar()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = m_pMainFrame->GetDocument();

  convChar = pDoc->convChar;
  repaint(false);
}
