#include "DoraCentralDocument.h"
#include "DoraCentralView.h"
#include "DoraMainFrame.h"
#include "DoraScrollView.h"

DoraRuntimeClass DoraScrollView::classDoraScrollView={"DoraScrollView", DoraScrollView::CreateView, NULL};

DoraClassObject *DoraScrollView::CreateView(QWidget *parent)
{
  return new DoraScrollView(parent);
}

DoraClassObject *DoraScrollView::CreateDocument()
{
  return NULL;
}

void DoraScrollView::OnDraw(QPainter *p)
{
  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->setBrush(QBrush(QColor(192, 192, 192)));
  p->drawRect(clientRect);
  
  m_pScrollBox->show();
  if(okScrollBar) {
    m_pVScrollBar->show();
  }
}

void DoraScrollView::OnSize(int cx, int cy)
{
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);

  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int wf = sizeFont.width();
  int hf = fm.height() - fm.descent();
  int x1 = clientRect.x();
  int y1 = clientRect.y();
  int x2, y2;
  if(okScrollBar) {
    x2 = clientRect.width() - (hf + 5);
    y2 = clientRect.height();
  } else {
    x2 = clientRect.width();
    y2 = clientRect.height();
  }
  scrollBoxRect.setX(x1);
  scrollBoxRect.setY(y1);
  scrollBoxRect.setWidth(x2);
  scrollBoxRect.setHeight(y2 - 2);
  m_pScrollBox->setGeometry(scrollBoxRect);
  m_pScrollBox->hide();

  x1 = clientRect.width() - (hf + 5);
  y1 = clientRect.y();
  x2 = hf + 4;
  vRectScrollBar.setX(x1);
  vRectScrollBar.setY(y1);
  vRectScrollBar.setWidth(x2);
  vRectScrollBar.setHeight(y2);
  m_pVScrollBar->setGeometry(vRectScrollBar);
  m_pVScrollBar->hide();

  x1 = clientRect.x();
  y1 = clientRect.height() - 21;
  x2 = clientRect.width() - 21;
  y2 = 20;
  hRectScrollBar.setX(x1);
  hRectScrollBar.setY(y1);
  hRectScrollBar.setWidth(x2);
  hRectScrollBar.setHeight(y2);
  m_pHScrollBar->setGeometry(hRectScrollBar);
  m_pHScrollBar->hide();
  
}

void DoraScrollView::OnCreate(DoraClassObject *pObject, DoraBaseView *pView)
{
  setFont(font);
  m_pObject = pObject;
  m_pView   = pView;
  m_pDialog = NULL;
  DoraCentralDocument *pDoc = GetDocument();

  DoraRuntimeClass *runtimeScrollBox = RUNTIME_CLASS(DoraScrollBox);
  DoraClassObject *objectScrollBox = runtimeScrollBox->CreateView(this);
  m_pScrollBox = dynamic_cast<DoraScrollBox *>(objectScrollBox);
  m_pScrollBox->convChar = convChar;
  m_pScrollBox->selectNewFont(pDoc->getDefaultFont(5));
  m_pScrollBox->maxRow = maxRow;
  m_pScrollBox->OnCreate(m_pObject, this);
  m_pScrollBox->hide();

  DoraRuntimeClass *runtimeVScrollBar = RUNTIME_CLASS(DoraVScrollBar);
  DoraClassObject *objectVScrollBar = runtimeVScrollBar->CreateView(this);
  m_pVScrollBar = dynamic_cast<DoraVScrollBar *>(objectVScrollBar);
  m_pVScrollBar->convChar = pDoc->convChar;
  m_pVScrollBar->selectNewFont(pDoc->getDefaultFont(5));
  m_pVScrollBar->OnCreate(m_pObject, this);
  m_pVScrollBar->hide();

  DoraRuntimeClass *runtimeHScrollBar = RUNTIME_CLASS(DoraHScrollBar);
  DoraClassObject *objectHScrollBar = runtimeHScrollBar->CreateView(this);
  m_pHScrollBar = dynamic_cast<DoraHScrollBar *>(objectHScrollBar);
  m_pHScrollBar->convChar = pDoc->convChar;
  m_pHScrollBar->selectNewFont(pDoc->getDefaultFont(5));
  m_pHScrollBar->OnCreate(m_pObject, this);
  m_pHScrollBar->hide();

}

//
void DoraScrollView::OnCreate(DoraClassObject *pObject, DoraBaseDialog *pDialog)
{
  setFont(font);
  m_pObject = pObject;
  m_pView   = NULL;
  m_pDialog = pDialog;
  DoraCentralDocument *pDoc = GetDocument();

  DoraRuntimeClass *runtimeScrollBox = RUNTIME_CLASS(DoraScrollBox);
  DoraClassObject *objectScrollBox = runtimeScrollBox->CreateView(this);
  m_pScrollBox = dynamic_cast<DoraScrollBox *>(objectScrollBox);
  m_pScrollBox->convChar = convChar;
  m_pScrollBox->selectNewFont(pDoc->getDefaultFont(5));
  m_pScrollBox->maxRow = maxRow;
  m_pScrollBox->OnCreate(m_pObject, this);
  m_pScrollBox->hide();

  DoraRuntimeClass *runtimeVScrollBar = RUNTIME_CLASS(DoraVScrollBar);
  DoraClassObject *objectVScrollBar = runtimeVScrollBar->CreateView(this);
  m_pVScrollBar = dynamic_cast<DoraVScrollBar *>(objectVScrollBar);
  m_pVScrollBar->convChar = pDoc->convChar;
  m_pVScrollBar->selectNewFont(pDoc->getDefaultFont(5));
  m_pVScrollBar->OnCreate(m_pObject, this);
  m_pVScrollBar->hide();

  DoraRuntimeClass *runtimeHScrollBar = RUNTIME_CLASS(DoraHScrollBar);
  DoraClassObject *objectHScrollBar = runtimeHScrollBar->CreateView(this);
  m_pHScrollBar = dynamic_cast<DoraHScrollBar *>(objectHScrollBar);
  m_pHScrollBar->convChar = pDoc->convChar;
  m_pHScrollBar->selectNewFont(pDoc->getDefaultFont(5));
  m_pHScrollBar->OnCreate(m_pObject, this);
  m_pHScrollBar->hide();
}

void DoraScrollView::addExpand(Str35 name, const int index = 1)
{
  m_pScrollBox->setDataElement(name, index);
}

void DoraScrollView::PostMouseMessage(int ctrl, int index)
{
    switch(ctrl) {
    case POST_SCROLLBARBUTTON_UP :
      m_pScrollBox->scrollUp();
      break;
    case POST_SCROLLBARBUTTON_DOWN :
      m_pScrollBox->scrollDown();
      break;
    default :
      if(m_pView != NULL)
	m_pView->PostMouseMessage(ctrl, index);
      if(m_pDialog != NULL)
	m_pDialog->PostMouseMessage(ctrl, index);
      break;
    }
}

DoraCentralDocument *DoraScrollView::GetDocument()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = m_pMainFrame->GetDocument();

  return pDoc;
}
