#include "DoraCentralDocument.h"
#include "DoraCentralView.h"
#include "DoraMainFrame.h"
#include "DoraSpinGroup.h"

DoraRuntimeClass DoraSpinGroup::classDoraSpinGroup={"DoraSpinGroup", DoraSpinGroup::CreateView, NULL};

DoraClassObject *DoraSpinGroup::CreateView(QWidget *parent)
{
  return new DoraSpinGroup(parent);
}

DoraClassObject *DoraSpinGroup::CreateDocument()
{
  return NULL;
}

void DoraSpinGroup::OnDraw(QPainter *p)
{
  p->setFont(font);
  p->setPen(QPen(QColor(0, 0, 0), 1));
  p->setBrush(QBrush(QColor(192, 192, 192)));
  p->drawRect(clientRect);
  QFontMetrics fm(font);
  QSize sizeFont = fm.size(SingleLine, "H", 1);
  int w = sizeFont.width();
  int maxElement = itemSpin.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    int x = itemSpin[i].rect.x();
    int y = itemSpin[i].rect.y() - 4;
    int len = strlen(caption[i].p);
    for(int j = 0; j <= len - 1; j++) {
      char *s = new char[2];
      s[0] = '\0';
      sprintf(s, "%c\0", caption[i].p[j]);
      QString qStr;
      qStr = "";qStr = convChar.getCharCodec(s);
      p->drawText(x, y, qStr);
      p->drawText(x+1,y, qStr);
      x += w;
      delete s;
    }
    itemSpin[i].spinBox->show();
  }
}

void DoraSpinGroup::OnSize(int cx, int cy)
{
  DoraCentralDocument *pDoc = GetDocument();
  clientRect.setX(0);
  clientRect.setY(0);
  clientRect.setWidth(cx);
  clientRect.setHeight(cy);
  QFontMetrics fm(pDoc->getDefaultFont(5));
  QSize fontSize = fm.size(SingleLine, "H", 1);
  int wFont = fontSize.width();
  int hFont = fm.height() - fm.descent();
  int w = (2 * (hFont + 4)) + (2 * wFont) + 5;
  int h = hFont + 4;
  int row = 2;
  int x = 5;
  int y = (row * h) + 5;
  int maxElement = itemSpin.maxElement();
  for(int i = 0; i <= maxElement - 1; i++) {
    y = (row * h) + 5;
    QRect rect;
    rect.setX(x);
    rect.setY(y);
    rect.setWidth(w);
    rect.setHeight(h);
    itemSpin[i].spinBox->setGeometry(rect);
    itemSpin[i].spinBox->hide();
    strSpin item;
    item = itemSpin[i];
    item.rect.setX(x);
    item.rect.setY(y);
    item.rect.setWidth(w);
    item.rect.setHeight(h);
    itemSpin.setElement(item, i + 1);
    row += 2;
  }
}

void DoraSpinGroup::OnCreate(DoraClassObject *pObject, DoraBaseView *pView)
{
  m_pObject = pObject;
  m_pView   = pView;
  setFont(font);

  DoraCentralDocument *pDoc = GetDocument();

  for(int i = 0; i <= 2; i++) {
    strSpin item;
    DoraRuntimeClass *runtimeSpin = RUNTIME_CLASS(DoraSpinBox);
    DoraClassObject *objectSpin = runtimeSpin->CreateView(this);
    item.spinBox = dynamic_cast<DoraSpinBox *>(objectSpin);
    item.spinBox->selectNewFont(pDoc->getDefaultFont(5));
    item.spinBox->convChar = pDoc->convChar;
    item.spinBox->OnCreate(m_pObject, this, 2);
    item.spinBox->ctrlID = DM_SPINCTRL;
    item.spinBox->nID = i + 1;
    if(i == 0)
      item.spinBox->setSpin(0, 23, 1);
    else
      item.spinBox->setSpin(0, 59, 1);
    item.spinBox->hide();
    itemSpin.setElement(item, i + 1);
  }
  caption[0] = "";caption[0] = pDoc->capSpin1;
  caption[1] = "";caption[1] = pDoc->capSpin2;
  caption[2] = "";caption[2] = pDoc->capSpin3;
}

DoraCentralDocument *DoraSpinGroup::GetDocument()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralView *m_pCentralView = m_pMainFrame->GetCentralView();
  DoraCentralDocument *pDoc = m_pCentralView->GetDocument();

  return pDoc;
}

void DoraSpinGroup::UpdateTimeView()
{
  DoraCentralDocument *pDoc = GetDocument();

  int h = pDoc->curentHour;
  int m = pDoc->curentMin;
  int s = pDoc->curentSec;

  itemSpin[0].spinBox->setDefaultValue(h);
  itemSpin[1].spinBox->setDefaultValue(m);
  itemSpin[2].spinBox->setDefaultValue(s);
}

void DoraSpinGroup::PostMouseMessage(int ctrl, int index)
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralView *m_pCentralView = m_pMainFrame->GetCentralView();
  DoraCentralDocument *pDoc = GetDocument();

  pDoc->curentHour = atoi(itemSpin[0].spinBox->getValue().p);
  pDoc->curentMin  = atoi(itemSpin[1].spinBox->getValue().p);
  pDoc->curentSec  = atoi(itemSpin[2].spinBox->getValue().p);

  m_pCentralView->stopClock();

}

void DoraSpinGroup::UpdateConvChar()
{
  DoraMainFrame *m_pMainFrame = dynamic_cast<DoraMainFrame *>(m_pObject);
  DoraCentralDocument *pDoc = m_pMainFrame->GetDocument();
  caption[0] = "";caption[0] = pDoc->capSpin1;
  caption[1] = "";caption[1] = pDoc->capSpin2;
  caption[2] = "";caption[2] = pDoc->capSpin3;
  repaint(false);
}
