#if !defined(_DYNACOLECT_H)
#define _DYNACOLECT_H

#include <stdio.h>
#include <stdlib.h>
#include "Typedef.h"

template <class T> class CList
{
  struct Node {
    T val;
    Node *next;
  };
  Node *Item, *tail, *head;

public:
  CList() {
    Item = head = tail = NULL;
    maxElement = 0;
  }

  void AddTail(T v) {
    Item = new Node;
    Item->val = v;
    if(head == 0) {
      head = tail = Item;
    } else {
      tail->next = Item;
    }
    tail = Item;
    tail->next = NULL;
    maxElement++;
  }

  void AddHead(T v) {
    Node *tmp = NULL;
    Item = new Node;
    Item->val = v;
    if(head == 0) {
      head = tail = Item;
      head->next = tail->next = NULL;
    } else {
      tmp = head;
      head = Item;
      head->next = tmp;
    }
    maxElement++;
  }

  void GetHead(T& v) {
    Item = head;
    v = Item->val;
  }

  void GetTail(T& v) {
    Item = tail;
    v = Item->val;
  }

  void RemoveHead() {
      Item = head;
      head = head->next;
      delete Item;
      maxElement--;
  }

private:
  int maxElement;

};

template <class T> class CDoubleList
{
  struct Node {
    T val;
    Node *prev, *next;
  };
  Node *Item, *head, *tail;
public:
  CDoubleList() {
    Item = head = tail = NULL;
  }

  void AddTail(T v) {
    Node *tmp = tail;
    Item = new Node;
    Item->val = v;
    if(tail != 0)
      tail->next = Item;
    else
      head = Item;
    tail = Item;
    tail->prev = tmp;
    tail->next = NULL;
  }
      
  void AddHead(T v) {
    Node *tmp = head;
    Item = new Node;
    Item->val = v;
    if(head != 0)
      head->prev = Item;
    else
      tail = Item;
    head = Item;
    head->next = tmp;
    head->prev = NULL;
  }
  
  void GetHead(T &v) {
    Item = head;
    v = Item->val;
  }

  void GetTail(T &v) {
    Item = tail;
    v = Item->val;
  }

  void RemoveTail() {
    Node *tmp = tail;
    tail = tmp->prev;
    if(tail != 0)
      tail->next = NULL;
    else
      head = NULL;
    delete tmp;
  }

  void RemoveHead() {
    Node *tmp = head;
    head = tmp->next;
    if(head != 0)
      head->prev = NULL;
    else
      tail = NULL;
    delete tmp;
  }
};

#define MAX_SIZE 1

template <class T> class DoraArray
{
public:
  DoraArray(int size = MAX_SIZE) {
    p = new T[size];
    maxSize = size;
  }
  
  ~DoraArray() {
    if(maxSize >= 1) {
      delete p;
      p = NULL;
    }
  }

  T operator[](const int index) {
    return p[index];
  }

  void swap(T x[], int ind1, int ind2) {
    T tmp;
    tmp     = x[ind1];
    x[ind1] = x[ind2];
    x[ind2] = tmp;
  }

  void addElement(T x) {
    int size = maxSize;
    T *tmp = new T[size+1];
    for(int i = 0; i <= size - 1; i++) {
      tmp[i] = p[i];
    }
    tmp[size] = x;

    delete p;
    p = new T[size+1];
    maxSize = size + 1;
    for(int i = 0; i <= size; i++) {
      p[i] = tmp[i];
    }
    delete tmp;
  }

  void setSize(const int newSize = 1) {
    if(maxSize <= 0) {
      p = new T[newSize];
      maxSize = newSize;
    } else {
      int size = maxSize + newSize;
      T *tmp = new T[size];
      for(int i = 0; i <= maxSize - 1; i++) {
	tmp[i] = p[i];
      }
      delete p;
      maxSize = size;
      p = new T[maxSize];
      for(int i = 0; i <= maxSize - 1; i++) {
	p[i] = tmp[i];
      }
      delete tmp;
    }
  }

  void setElement(T x, const int index = 1) {
    if(maxSize <= 0) {
      int count = index;
      p = new T[count];
      p[index - 1] = NULL;
      p[index - 1] = x;
      maxSize = count;
    } else {
      if(index > maxSize) {
	int count = (index - maxSize) + 1;
	int size = (maxSize + count) - 1;
	T *tmp = new T[size];
	for(int i = 0; i <= maxSize - 1; i++) {
	  tmp[i] = NULL;
	  tmp[i] = p[i];
	}
	tmp[index - 1] = x;
	delete p;
	p = new T[size];
	for(int i = 0; i <= size - 1; i++) {
	  p[i] = NULL;
	  p[i] = tmp[i];
	}
	maxSize = size;
	delete tmp;
      } else {
	p[index - 1] = NULL;
	p[index - 1] = x;
      }
    }
  }

  void insertElement(const int index) {
    if((index >= 1) && (index <= maxSize - 1)) {
      T *tmp = new T[maxSize + 1];
      int j = 0;
      for(int i = 0; i <= maxSize - 1; i++) {
	if(i == index - 1) {
	  j++;
	}
	tmp[j] = p[i];j++;
      }
      delete p;
      p = new T[maxSize + 1];
      maxSize++;
      for(int i = 0; i <= maxSize - 1; i++) {
	p[i] = tmp[i];
      }
      delete tmp;
    }  
  }

  void insertElement(T x, const int index) {
    if((index >= 1) && (index <= maxSize - 1)) {
      T *tmp = new T[maxSize + 1];
      int j = 0;
      for(int i = 0; i <= maxSize - 1; i++) {
	if(i == index - 1) {
	  tmp[j] = x;j++;
	}
	tmp[j] = p[i];j++;
      }
      delete p;
      p = new T[maxSize + 1];
      maxSize++;
      for(int i = 0; i <= maxSize - 1; i++) {
	p[i] = tmp[i];
      }
      delete tmp;
    }  
  }

  int maxElement() {
    return maxSize;
  }

  void removeElement(const int index) {
    if(maxSize < 1) {
      maxSize = 0;
      delete p;
    } else {
      if((index >= 1) && (index <= maxSize)) {
	T *tmp = new T[maxSize];
	int j = 0;
	for(int i = 0; i <= maxSize - 1; i++) {
	  if(i == index - 1) 
	    j++;
	  if(j <= maxSize - 1) {
	    tmp[i] = NULL;
	    tmp[i] = p[j];j++;
	  }
	}
	maxSize--;
	if(maxSize >= 1) {
	  delete p;
	  p = new T[maxSize];
	  for(int i = 0; i <= maxSize - 1; i++) {
	    p[i] = NULL;
	    p[i] = tmp[i];
	  }
	}
	delete tmp;
      }
    }
  }

  void removeAllElement() {
    if(maxSize >= 1) {
      do {
	int index = 1;
	removeElement(index);
      } while(maxSize != 0);
    }
  }

public:
  T *p;
  int maxSize;

};


struct Cols {
public:
  CString cell;

  Cols(Cols *x = NULL) {
    cell = "";
  }
  
  Cols(const Cols &x) {
    cell = "";
    cell = x.cell;
  }

  Cols &operator=(const Cols &x) {
    cell = "";
    cell = x.cell;
    return *this;
  }
};

typedef DoraArray<Cols> TypeItemColl;
typedef DoraArray<DataRecord> TypeDataRecord;

class ColsItem {
public:
  ColsItem();
  ~ColsItem();

public:
  void    addColumn(const int count);
  void    insertColumn(int index);
  void    insertColumn(int index, const int count);
  void    removeColumn(int index);
  void    removeAllColumn();
  void    setDataColumn(Cols x, int index);
  void    setDataColumn(CString s, int index);
  CString getDataColumn(int index);
  int     maxColumn();

public:
  TypeItemColl itemColl;

private:
  int maxColumns;

};

struct DoraColumn {
public:
  ColsItem *collItem;

  DoraColumn(DoraColumn *x = NULL) {
    collItem = NULL;
  }

  DoraColumn(const DoraColumn &x) {
    collItem = NULL;
    collItem = x.collItem;
  }

  DoraColumn &operator=(const DoraColumn &x) {
    collItem = NULL;
    collItem = x.collItem;
    return *this;
  }
};

typedef DoraArray<DoraColumn> TypeDoraColumn;

class DoraTable {
public:
  TypeDoraColumn rowItem;

public:
  DoraTable();
  ~DoraTable();

public:
  void    addColumns(const int countCol);
  void    addRow(const int countRow);
  void    setData(CString s, int rowIndex, int colIndex);
  CString getData(int rowIndex, int colIndex);
  void    insertColumn(int index, const int countCol);
  void    insertRow(int index, const int countRow);
  void    removeColumn(int index, const int countCol);
  void    removeRow(int index, const int countRow);
  void    removeAllRow();
  int     maxRows();
  int     maxCol();

private:
  int maxRow;

};


struct ItemTree {
public:
  DataRecord Data;
  TypeDataRecord ItemData;

  ItemTree(ItemTree *x = NULL) {
    ItemData.removeAllElement();
    Data.DelNo = '\0';
    Data.RecNo = 0;
    for(int i = 0; i <= 19; i++)
      Data.Pole[i] = "";
  }

  ItemTree(ItemTree &x) {
    DataRecord iData;
    ItemData.removeAllElement();
    Data.DelNo = x.Data.DelNo;
    Data.RecNo = x.Data.RecNo;
    for(int i = 0; i <= 19; i++) {
      Data.Pole[i] = "";
      Data.Pole[i] = x.Data.Pole[i];
    }
    for(int i = 0; i <= x.ItemData.maxElement() - 1; i++) {
      for(int j = 0; j <= 19; j++) {
	iData.Pole[j] = "";
	iData.Pole[j] = x.ItemData[i].Pole[j];
      }
      ItemData.setElement(iData, i + 1);
    }
  }

  ItemTree &operator=(ItemTree x) {
    DataRecord iData;
    ItemData.removeAllElement();
    Data.DelNo = x.Data.DelNo;
    Data.RecNo = x.Data.RecNo;
    for(int i = 0; i <= 19; i++) {
      Data.Pole[i] = "";
      Data.Pole[i] = x.Data.Pole[i];
    }
    for(int i = 0; i <= x.ItemData.maxElement() - 1; i++) {
      for(int j = 0; j <= 19; i++) {
	iData.Pole[j] = "";
	iData.Pole[j] = x.ItemData[i].Pole[j];
      }
      ItemData.setElement(iData, i + 1);
    }
    return *this;
  }
};



#endif













